/*
 * AutomatonCompare.h
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#ifndef AUTOMATON_COMPARE_H_
#define AUTOMATON_COMPARE_H_

#include <core/multipleDispatch.hpp>

#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>
#include <set>
#include <list>
#include <map>
#include <utility>
#include <vector>
#include <iostream>
#include <ostream>
#include <algorithm>

#include "automaton/FSM/DFA.h"
#include "automaton/FSM/NFA.h"
#include "automaton/FSM/MultiInitialStateNFA.h"
#include "automaton/FSM/EpsilonNFA.h"
#include "automaton/FSM/ExtendedNFA.h"
#include "automaton/FSM/CompactNFA.h"
#include "automaton/TA/DFTA.h"
#include "automaton/TA/NFTA.h"
#include "automaton/PDA/NPDA.h"
#include "automaton/PDA/DPDA.h"
#include "automaton/PDA/InputDrivenNPDA.h"
#include "automaton/PDA/InputDrivenDPDA.h"
#include "automaton/PDA/VisiblyPushdownNPDA.h"
#include "automaton/PDA/VisiblyPushdownDPDA.h"
#include "automaton/PDA/RealTimeHeightDeterministicNPDA.h"
#include "automaton/PDA/RealTimeHeightDeterministicDPDA.h"
#include "automaton/PDA/SinglePopNPDA.h"
#include "automaton/PDA/SinglePopDPDA.h"
#include "automaton/TM/OneTapeDTM.h"

class AutomatonCompare : public std::DoubleDispatch<AutomatonCompare, int, const automaton::AutomatonBase &, const automaton::AutomatonBase &> {
private:
	template<class SymbolType, class StateType>
	static bool testCompare(const automaton::DFA < SymbolType, StateType > & a, const automaton::DFA < SymbolType, StateType > & b);
	template<class SymbolType, class StateType>
	static void printCompare(const automaton::DFA < SymbolType, StateType > & a, const automaton::DFA < SymbolType, StateType > & b);

	template<class SymbolType, class StateType>
	static bool testCompare(const automaton::NFA < SymbolType, StateType > & a, const automaton::NFA < SymbolType, StateType > & b);
	template<class SymbolType, class StateType>
	static void printCompare(const automaton::NFA < SymbolType, StateType > & a, const automaton::NFA < SymbolType, StateType > & b);

	template<class SymbolType, class StateType>
	static bool testCompare(const automaton::MultiInitialStateNFA < SymbolType, StateType > & a, const automaton::MultiInitialStateNFA < SymbolType, StateType > & b);
	template<class SymbolType, class StateType>
	static void printCompare(const automaton::MultiInitialStateNFA < SymbolType, StateType > & a, const automaton::MultiInitialStateNFA < SymbolType, StateType > & b);

	template<class SymbolType, class EpsilonType, class StateType>
	static bool testCompare(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & a, const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & b);
	template<class SymbolType, class EpsilonType, class StateType>
	static void printCompare(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & a, const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & b);

	template<class SymbolType, class StateType>
	static bool testCompare(const automaton::ExtendedNFA < SymbolType, StateType > & a, const automaton::ExtendedNFA < SymbolType, StateType > & b);
	template<class SymbolType, class StateType>
	static void printCompare(const automaton::ExtendedNFA < SymbolType, StateType > & a, const automaton::ExtendedNFA < SymbolType, StateType > & b);

	template<class SymbolType, class StateType>
	static bool testCompare(const automaton::CompactNFA < SymbolType, StateType > & a, const automaton::CompactNFA < SymbolType, StateType > & b);
	template<class SymbolType, class StateType>
	static void printCompare(const automaton::CompactNFA < SymbolType, StateType > & a, const automaton::CompactNFA < SymbolType, StateType > & b);

	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);

	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);

	template<class SymbolType, class RankType, class StateType>
	static bool testCompare(const automaton::DFTA < SymbolType, RankType, StateType > & a, const automaton::DFTA < SymbolType, RankType, StateType > & b);
	template<class SymbolType, class RankType, class StateType>
	static void printCompare(const automaton::DFTA < SymbolType, RankType, StateType > & a, const automaton::DFTA < SymbolType, RankType, StateType > & b);

	template<class SymbolType, class RankType, class StateType>
	static bool testCompare(const automaton::NFTA < SymbolType, RankType, StateType > & a, const automaton::NFTA < SymbolType, RankType, StateType > & b);
	template<class SymbolType, class RankType, class StateType>
	static void printCompare(const automaton::NFTA < SymbolType, RankType, StateType > & a, const automaton::NFTA < SymbolType, RankType, StateType > & b);

	template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b);

	template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::InputDrivenNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::InputDrivenNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::InputDrivenNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::InputDrivenNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b);

	template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b);

	template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b);

	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);

	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);

	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);

	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static bool testCompare(const automaton::SinglePopNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::SinglePopNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);
	template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
	static void printCompare(const automaton::SinglePopNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::SinglePopNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b);

	template<class SymbolType, class StateType>
	static bool testCompare(const automaton::OneTapeDTM < SymbolType, StateType > & a, const automaton::OneTapeDTM < SymbolType, StateType > & b);
	template<class SymbolType, class StateType>
	static void printCompare(const automaton::OneTapeDTM < SymbolType, StateType > & a, const automaton::OneTapeDTM < SymbolType, StateType > & b);

	template <class T> static void setCompare(const std::set<T> &a, const std::set<T> &b);
	template <class T> static void listCompare(const std::list<T> &a, const std::list<T> &b);
	template <class T, class R> static void mapCompare(const std::map<T, R> &a, const std::map<T, R> &b);
public:
	template<class T>
	static int compare(const T & a, const T & b);

	static int compare(const automaton::Automaton& a, const automaton::Automaton& b);
};

template<class SymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::DFA < SymbolType, StateType > & a, const automaton::DFA < SymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class SymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::MultiInitialStateNFA < SymbolType, StateType > & a, const automaton::MultiInitialStateNFA < SymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialStates()  == b.getInitialStates()  &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class SymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::NFA < SymbolType, StateType > & a, const automaton::NFA < SymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class SymbolType, class EpsilonType, class StateType>
bool AutomatonCompare::testCompare(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & a, const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class SymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::ExtendedNFA < SymbolType, StateType > & a, const automaton::ExtendedNFA < SymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class SymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::CompactNFA < SymbolType, StateType > & a, const automaton::CompactNFA < SymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class SymbolType, class RankType, class StateType>
bool AutomatonCompare::testCompare(const automaton::DFTA < SymbolType, RankType, StateType > & a, const automaton::DFTA < SymbolType, RankType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class SymbolType, class RankType, class StateType>
bool AutomatonCompare::testCompare(const automaton::NFTA < SymbolType, RankType, StateType > & a, const automaton::NFTA < SymbolType, RankType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
//			a.getPushdownStoreAlphabet()  == b.getPushdownStoreAlphabet()  &&
			a.getInitialSymbol()  == b.getInitialSymbol()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
//			a.getPushdownStoreAlphabet()  == b.getPushdownStoreAlphabet()  &&
			a.getInitialSymbol()  == b.getInitialSymbol()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()             == b.getFinalStates()             &&
			a.getInitialState()            == b.getInitialState()            &&
//			a.getInputAlphabet()           == b.getInputAlphabet()           &&
//			a.getPushdownStoreAlphabet()   == b.getPushdownStoreAlphabet()           &&
			a.getInitialSymbol()           == b.getInitialSymbol()           &&
			a.getStates()                  == b.getStates()                  &&
			a.getPushdownStoreOperations() == b.getPushdownStoreOperations() &&
			a.getTransitions()             == b.getTransitions()             ;
}

template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::InputDrivenNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::InputDrivenNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()             == b.getFinalStates()             &&
			a.getInitialState()            == b.getInitialState()            &&
//			a.getInputAlphabet()           == b.getInputAlphabet()           &&
//			a.getPushdownStoreAlphabet()   == b.getPushdownStoreAlphabet()           &&
			a.getInitialSymbol()           == b.getInitialSymbol()           &&
			a.getStates()                  == b.getStates()                  &&
			a.getPushdownStoreOperations() == b.getPushdownStoreOperations() &&
			a.getTransitions()             == b.getTransitions()             ;
}

template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()            == b.getFinalStates()            &&
			a.getInitialState()           == b.getInitialState()           &&
//			a.getCallInputAlphabet()      == b.getCallInputAlphabet()      &&
//			a.getReturnnputAlphabet()     == b.getReturnInputAlphabet()    &&
//			a.getLocalInputAlphabet()     == b.getLocalInputAlphabet()     &&
//			a.getPushdownStoreAlphabet()  == b.getPushdownStoreAlphabet()          &&
			a.getBottomOfTheStackSymbol() == b.getBottomOfTheStackSymbol() &&
			a.getStates()                 == b.getStates()                 &&
			a.getCallTransitions()        == b.getCallTransitions()        &&
			a.getReturnTransitions()      == b.getReturnTransitions()      &&
			a.getLocalTransitions()       == b.getLocalTransitions()       ;
}

template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()            == b.getFinalStates()            &&
			a.getInitialStates()          == b.getInitialStates()          &&
//			a.getCallInputAlphabet()      == b.getCallInputAlphabet()      &&
//			a.getReturnnputAlphabet()     == b.getReturnInputAlphabet()    &&
//			a.getLocalInputAlphabet()     == b.getLocalInputAlphabet()     &&
//			a.getPushdownStoreAlphabet()  == b.getPushdownStoreAlphabet()          &&
			a.getBottomOfTheStackSymbol() == b.getBottomOfTheStackSymbol() &&
			a.getStates()                 == b.getStates()                 &&
			a.getCallTransitions()        == b.getCallTransitions()        &&
			a.getReturnTransitions()      == b.getReturnTransitions()      &&
			a.getLocalTransitions()       == b.getLocalTransitions()       ;
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()            == b.getFinalStates()            &&
			a.getInitialState()           == b.getInitialState()           &&
//			a.getInputAlphabet()          == b.getInputAlphabet()          &&
//			a.getPushdownStoreAlphabet()  == b.getPushdownStoreAlphabet()          &&
			a.getBottomOfTheStackSymbol() == b.getBottomOfTheStackSymbol() &&
			a.getStates()                 == b.getStates()                 &&
			a.getCallTransitions()        == b.getCallTransitions()        &&
			a.getReturnTransitions()      == b.getReturnTransitions()      &&
			a.getLocalTransitions()       == b.getLocalTransitions()       ;
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()            == b.getFinalStates()            &&
			a.getInitialStates()          == b.getInitialStates()          &&
//			a.getInputAlphabet()          == b.getInputAlphabet()          &&
//			a.getPushdownStoreAlphabet()  == b.getPushdownStoreAlphabet()          &&
			a.getBottomOfTheStackSymbol() == b.getBottomOfTheStackSymbol() &&
			a.getStates()                 == b.getStates()                 &&
			a.getCallTransitions()        == b.getCallTransitions()        &&
			a.getReturnTransitions()      == b.getReturnTransitions()      &&
			a.getLocalTransitions()       == b.getLocalTransitions()       ;
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
//			a.getPushdownStoreAlphabet()  == b.getPushdownStoreAlphabet()  &&
			a.getInitialSymbol()  == b.getInitialSymbol()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::SinglePopNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::SinglePopNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
//			a.getPushdownStoreAlphabet()  == b.getPushdownStoreAlphabet()  &&
			a.getInitialSymbol()  == b.getInitialSymbol()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

template<class SymbolType, class StateType>
bool AutomatonCompare::testCompare(const automaton::OneTapeDTM < SymbolType, StateType > & a, const automaton::OneTapeDTM < SymbolType, StateType > & b) {
	return  	a.getBlankSymbol()    == b.getBlankSymbol()    &&
			a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
//			a.getTapeAlphabet()   == b.getTapeAlphabet()   &&
			a.getTransitions()    == b.getTransitions()    ;
}

template <class T> void AutomatonCompare::setCompare(const std::set<T> &a, const std::set<T> &b) {
	std::set<T> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::set<T> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::set<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << *iter << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::set<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << *iter << std::endl;
	}
}

template <class T> void AutomatonCompare::listCompare(const std::list<T> &a, const std::list<T> &b) {
	std::list<T> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::list<T> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::list<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << *iter << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::list<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << *iter << std::endl;
	}
}

template <class T, class R> void AutomatonCompare::mapCompare(const std::map<T, R> &a, const std::map<T, R> &b) {
	std::map<T, R> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::map<T, R> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::map<T, R>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << iter->first << ", " << iter->second << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::map<T, R>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << iter->first << ", " << iter->second << std::endl;
	}
}

template<class SymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::DFA < SymbolType, StateType > & a, const automaton::DFA < SymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class SymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::MultiInitialStateNFA < SymbolType, StateType > & a, const automaton::MultiInitialStateNFA < SymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()) {
		std::cout << "InitialStates" << std::endl;

		AutomatonCompare::setCompare(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class SymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::NFA < SymbolType, StateType > & a, const automaton::NFA < SymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class SymbolType, class EpsilonType, class StateType>
void AutomatonCompare::printCompare(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & a, const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class SymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::ExtendedNFA < SymbolType, StateType > & a, const automaton::ExtendedNFA < SymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class SymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::CompactNFA < SymbolType, StateType > & a, const automaton::CompactNFA < SymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class SymbolType, class RankType, class StateType>
void AutomatonCompare::printCompare(const automaton::DFTA < SymbolType, RankType, StateType > & a, const automaton::DFTA < SymbolType, RankType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class SymbolType, class RankType, class StateType>
void AutomatonCompare::printCompare(const automaton::NFTA < SymbolType, RankType, StateType > & a, const automaton::NFTA < SymbolType, RankType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()) {
		std::cout << "InitialSymbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::NPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()) {
		std::cout << "InitialSymbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()) {
		std::cout << "InitialSymbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getPushdownStoreOperations() != b.getPushdownStoreOperations()) {
		std::cout << "PushdownStoreOperations" << std::endl;

		AutomatonCompare::mapCompare(a.getPushdownStoreOperations(), b.getPushdownStoreOperations());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::InputDrivenNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::InputDrivenNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()) {
		std::cout << "InitialSymbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getPushdownStoreOperations() != b.getPushdownStoreOperations()) {
		std::cout << "PushdownStoreOperations" << std::endl;

		AutomatonCompare::mapCompare(a.getPushdownStoreOperations(), b.getPushdownStoreOperations());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getCallInputAlphabet() != b.getCallInputAlphabet()) {
		std::cout << "CallInputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getCallInputAlphabet(), b.getCallInputAlphabet());
	}

	if(a.getReturnInputAlphabet() != b.getReturnInputAlphabet()) {
		std::cout << "ReturnInputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getReturnInputAlphabet(), b.getReturnInputAlphabet());
	}

	if(a.getLocalInputAlphabet() != b.getLocalInputAlphabet()) {
		std::cout << "LocalInputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getLocalInputAlphabet(), b.getLocalInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getBottomOfTheStackSymbol() != b.getBottomOfTheStackSymbol()) {
		std::cout << "BottomOfTheStackSymbol" << std::endl;

		std::cout << "< " << a.getBottomOfTheStackSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getBottomOfTheStackSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getCallTransitions() != b.getCallTransitions()) {
		std::cout << "CallTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getCallTransitions(), b.getCallTransitions());
	}

	if(a.getReturnTransitions() != b.getReturnTransitions()) {
		std::cout << "ReturnTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getReturnTransitions(), b.getReturnTransitions());
	}

	if(a.getLocalTransitions() != b.getLocalTransitions()) {
		std::cout << "LocalTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getLocalTransitions(), b.getLocalTransitions());
	}
}

template<class InputSymbolType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & a, const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()) {
		std::cout << "Initial states" << std::endl;

		AutomatonCompare::setCompare(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getCallInputAlphabet() != b.getCallInputAlphabet()) {
		std::cout << "CallInputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getCallInputAlphabet(), b.getCallInputAlphabet());
	}

	if(a.getReturnInputAlphabet() != b.getReturnInputAlphabet()) {
		std::cout << "ReturnInputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getReturnInputAlphabet(), b.getReturnInputAlphabet());
	}

	if(a.getLocalInputAlphabet() != b.getLocalInputAlphabet()) {
		std::cout << "LocalInputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getLocalInputAlphabet(), b.getLocalInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getBottomOfTheStackSymbol() != b.getBottomOfTheStackSymbol()) {
		std::cout << "BottomOfTheStackSymbol" << std::endl;

		std::cout << "< " << a.getBottomOfTheStackSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getBottomOfTheStackSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getCallTransitions() != b.getCallTransitions()) {
		std::cout << "CallTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getCallTransitions(), b.getCallTransitions());
	}

	if(a.getReturnTransitions() != b.getReturnTransitions()) {
		std::cout << "ReturnTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getReturnTransitions(), b.getReturnTransitions());
	}

	if(a.getLocalTransitions() != b.getLocalTransitions()) {
		std::cout << "LocalTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getLocalTransitions(), b.getLocalTransitions());
	}
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getBottomOfTheStackSymbol() != b.getBottomOfTheStackSymbol()) {
		std::cout << "BottomOfTheStackSymbol" << std::endl;

		std::cout << "< " << a.getBottomOfTheStackSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getBottomOfTheStackSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getCallTransitions() != b.getCallTransitions()) {
		std::cout << "CallTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getCallTransitions(), b.getCallTransitions());
	}

	if(a.getReturnTransitions() != b.getReturnTransitions()) {
		std::cout << "ReturnTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getReturnTransitions(), b.getReturnTransitions());
	}

	if(a.getLocalTransitions() != b.getLocalTransitions()) {
		std::cout << "LocalTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getLocalTransitions(), b.getLocalTransitions());
	}
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::RealTimeHeightDeterministicNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()) {
		std::cout << "Initial states" << std::endl;

		AutomatonCompare::setCompare(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getBottomOfTheStackSymbol() != b.getBottomOfTheStackSymbol()) {
		std::cout << "BottomOfTheStackSymbol" << std::endl;

		std::cout << "< " << a.getBottomOfTheStackSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getBottomOfTheStackSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getCallTransitions() != b.getCallTransitions()) {
		std::cout << "CallTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getCallTransitions(), b.getCallTransitions());
	}

	if(a.getReturnTransitions() != b.getReturnTransitions()) {
		std::cout << "ReturnTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getReturnTransitions(), b.getReturnTransitions());
	}

	if(a.getLocalTransitions() != b.getLocalTransitions()) {
		std::cout << "LocalTransitions" << std::endl;

		AutomatonCompare::mapCompare(a.getLocalTransitions(), b.getLocalTransitions());
	}
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::SinglePopDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()) {
		std::cout << "InitialSymbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::SinglePopNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & a, const automaton::SinglePopNPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getPushdownStoreAlphabet() != b.getPushdownStoreAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getPushdownStoreAlphabet(), b.getPushdownStoreAlphabet());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()) {
		std::cout << "InitialSymbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template<class SymbolType, class StateType>
void AutomatonCompare::printCompare(const automaton::OneTapeDTM < SymbolType, StateType > & a, const automaton::OneTapeDTM < SymbolType, StateType > & b) {
	std::cout << "AutomatonCompareer" << std::endl;

	if(a.getBlankSymbol() != b.getBlankSymbol()) {
		std::cout << "Blank symbol" << std::endl;

		std::cout << "< " << a.getBlankSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getBlankSymbol() << std::endl;
	}

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonCompare::setCompare(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonCompare::setCompare(a.getStates(), b.getStates());
	}

	if(a.getTapeAlphabet() != b.getTapeAlphabet()) {
		std::cout << "TapeAlphabet" << std::endl;

		AutomatonCompare::setCompare(a.getTapeAlphabet(), b.getTapeAlphabet());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonCompare::mapCompare(a.getTransitions(), b.getTransitions());
	}
}

template < class T>
int AutomatonCompare::compare(const T & a, const T & b) {
	if(!AutomatonCompare::testCompare(a, b)) {
	  AutomatonCompare::printCompare(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

#endif /* AUTOMATON_COMPARE_H_ */
