/*
 * GrammarCompare.h
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#ifndef GRAMMAR_COMPARE_H_
#define GRAMMAR_COMPARE_H_

#include <core/multipleDispatch.hpp>
#include <ostream>

#include <grammar/Grammar.h>
#include <grammar/GrammarFeatures.h>
#include <set>
#include <list>
#include <map>
#include <utility>

class GrammarCompare : public std::DoubleDispatch<GrammarCompare, int, const grammar::GrammarBase &, const grammar::GrammarBase &> {
private:
	static bool testCompare(const grammar::LeftLG < > & a, const grammar::LeftLG < > & b);
	static void printCompare(const grammar::LeftLG < > & a, const grammar::LeftLG < > & b);

	static bool testCompare(const grammar::LeftRG < > & a, const grammar::LeftRG < > & b);
	static void printCompare(const grammar::LeftRG < > & a, const grammar::LeftRG < > & b);

	static bool testCompare(const grammar::RightLG < > & a, const grammar::RightLG < > & b);
	static void printCompare(const grammar::RightLG < > & a, const grammar::RightLG < > & b);

	static bool testCompare(const grammar::RightRG < > & a, const grammar::RightRG < > & b);
	static void printCompare(const grammar::RightRG < > & a, const grammar::RightRG < > & b);

	static bool testCompare(const grammar::LG < > & a, const grammar::LG < > & b);
	static void printCompare(const grammar::LG < > & a, const grammar::LG < > & b);

	static bool testCompare(const grammar::CFG < > & a, const grammar::CFG < > & b);
	static void printCompare(const grammar::CFG < > & a, const grammar::CFG < > & b);

	static bool testCompare(const grammar::EpsilonFreeCFG < > & a, const grammar::EpsilonFreeCFG < > & b);
	static void printCompare(const grammar::EpsilonFreeCFG < > & a, const grammar::EpsilonFreeCFG < > & b);

	static bool testCompare(const grammar::CNF < > & a, const grammar::CNF < > & b);
	static void printCompare(const grammar::CNF < > & a, const grammar::CNF < > & b);

	static bool testCompare(const grammar::GNF < > & a, const grammar::GNF < > & b);
	static void printCompare(const grammar::GNF < > & a, const grammar::GNF < > & b);

	static bool testCompare(const grammar::CSG < > & a, const grammar::CSG < > & b);
	static void printCompare(const grammar::CSG < > & a, const grammar::CSG < > & b);

	static bool testCompare(const grammar::NonContractingGrammar < > & a, const grammar::NonContractingGrammar < > & b);
	static void printCompare(const grammar::NonContractingGrammar < > & a, const grammar::NonContractingGrammar < > & b);

	static bool testCompare(const grammar::ContextPreservingUnrestrictedGrammar < > & a, const grammar::ContextPreservingUnrestrictedGrammar < > & b);
	static void printCompare(const grammar::ContextPreservingUnrestrictedGrammar < > & a, const grammar::ContextPreservingUnrestrictedGrammar < > & b);

	static bool testCompare(const grammar::UnrestrictedGrammar < > & a, const grammar::UnrestrictedGrammar < > & b);
	static void printCompare(const grammar::UnrestrictedGrammar < > & a, const grammar::UnrestrictedGrammar < > & b);

	template <class T> static void setCompare(const std::set<T> &a, const std::set<T> &b);
	template <class T> static void listCompare(const std::list<T> &a, const std::list<T> &b);
	template <class T, class R> static void mapCompare(const std::map<T, R> &a, const std::map<T, R> &b);
public:
	template < class T >
	static int compare(const T & a, const T & b);

	static int compare(const grammar::Grammar& a, const grammar::Grammar& b);
};

template < class T >
int GrammarCompare::compare(const T & a, const T & b) {
	if(!GrammarCompare::testCompare(a, b)) {
	  GrammarCompare::printCompare(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

#endif /* GRAMMAR_COMPARE_H_ */
