/*
 * acompare.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <global/GlobalData.h>
#include <measure>
#include <sax/FromXMLParserHelper.h>
#include <tclap/CmdLine.h>

#include "exception/CommonException.h"
#include "factory/XmlDataFactory.hpp"

#include "AutomatonCompare.h"
#include "GrammarCompare.h"
#include "StringCompare.h"

int main ( int argc, char * * argv ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Regexp derivative compute binary", ' ', "0.01" );

		TCLAP::UnlabeledValueArg < std::string > input1 ( "input1", "The first input to compare", true, "-", "file" );
		cmd.add ( input1 );

		TCLAP::UnlabeledValueArg < std::string > input2 ( "input2", "The second input to compare", true, "-", "file" );
		cmd.add ( input2 );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		std::deque < sax::Token > tokens1 = sax::FromXMLParserHelper::parseInput ( input1 );
		std::deque < sax::Token > tokens2 = sax::FromXMLParserHelper::parseInput ( input2 );

		int res;

		if ( alib::XmlDataFactory::first < automaton::Automaton > ( tokens1 ) && alib::XmlDataFactory::first < automaton::Automaton > ( tokens2 ) ) {
			automaton::Automaton automaton1 = alib::XmlDataFactory::fromTokens ( std::move ( tokens1 ) );
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens ( std::move ( tokens2 ) );

			measurements::end ( );
			measurements::start ( "Compare", measurements::Type::MAIN );

			res = AutomatonCompare::compare ( automaton1, automaton2 );
		} else if ( alib::XmlDataFactory::first < grammar::Grammar > ( tokens1 ) && alib::XmlDataFactory::first < grammar::Grammar > ( tokens2 ) ) {
			grammar::Grammar grammar1 = alib::XmlDataFactory::fromTokens ( std::move ( tokens1 ) );
			grammar::Grammar grammar2 = alib::XmlDataFactory::fromTokens ( std::move ( tokens2 ) );

			measurements::end ( );
			measurements::start ( "Compare", measurements::Type::MAIN );

			res = GrammarCompare::compare ( grammar1, grammar2 );
		} else if ( alib::XmlDataFactory::first < string::String > ( tokens1 ) && alib::XmlDataFactory::first < string::String > ( tokens2 ) ) {
			string::String string1 = alib::XmlDataFactory::fromTokens ( std::move ( tokens1 ) );
			string::String string2 = alib::XmlDataFactory::fromTokens ( std::move ( tokens2 ) );

			measurements::end ( );
			measurements::start ( "Compare", measurements::Type::MAIN );

			res = StringCompare::compare ( string1, string2 );
		} else {
			throw exception::CommonException ( "Only automata, grammars and strings can be compared." );
		}

		measurements::end ( );
		measurements::end ( );

		return res;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cerr << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
