/*
 * NFACommon.h
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#ifndef NFA_COMMON_H_
#define NFA_COMMON_H_

#include <set>
#include <common/DefaultStateType.h>

namespace automaton {

namespace determinize {

/**
 * Returns existing state from the resulting automaton, if there is one, or creates new one and adds it into
 *  the resulting deterministic automaton.
 *
 * @param originalStates set of states from nondeterministic fsm which represents state of deterministic fsm
 * @return state of deterministic fsm
 */
DefaultStateType createDFAState ( std::set < DefaultStateType > nfaStates );

/**
 * Finds states from nondeterministic fsm to which at least one state from given set of states have transition
 *  with given input.
 *
 * @param fromStates set of states from nondeterministic fsm
 * @param input symbol from input alphabet
 * @return set of states from nondeterministic fsm
 */
const std::set < DefaultStateType > & recreateNFAStates ( const DefaultStateType & dfaState );

} /* namespace determinize */

} /* namespace automaton */

#endif /* NFA_COMMON_H_ */
