#ifndef RHDPDA_VPA_COMMON_H_
#define RHDPDA_VPA_COMMON_H_

#include "common/DefaultStateType.h"
#include "common/DefaultSymbolType.h"
#include "common/DefaultEpsilonType.h"
#include <variant>
#include <set>
#include <map>
#include "automaton/PDA/VisiblyPushdownDPDA.h"
#include "automaton/PDA/RealTimeHeightDeterministicDPDA.h"

namespace automaton {

namespace determinize {

DefaultStateType packToStateLabel(std::set<std::pair<DefaultStateType, DefaultStateType>>&& data);

const std::set<std::pair<DefaultStateType, DefaultStateType>> & unpackFromStateLabel(const DefaultStateType& data);

DefaultSymbolType packToStackSymbolLabel(std::pair<DefaultStateType, DefaultSymbolType>&& data);

const std::pair<DefaultStateType, DefaultSymbolType> & unpackFromDVPAStackSymbol(const DefaultSymbolType& symbol);

DefaultSymbolType packToStackSymbolLabel(std::pair<DefaultStateType, std::variant<DefaultEpsilonType, DefaultSymbolType>>&& data);

const std::pair<DefaultStateType, std::variant<DefaultEpsilonType, DefaultSymbolType>> & unpackFromDRHDPDAStackSymbol(const DefaultSymbolType& symbol);

std::set<DefaultStateType> retrieveDSubSet(const std::set<std::pair<DefaultStateType, DefaultStateType>>& localOperation);

std::set<std::pair<DefaultStateType, DefaultStateType>> createIdentity(const std::set<DefaultStateType>& states);

template<class T, class S, class R>
std::set<DefaultStateType> existsDirtyState(const T& d, const std::set<DefaultStateType>& rubbishStates, const std::map<std::pair<DefaultStateType, S>, std::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions, const std::map<std::pair<DefaultStateType, S>, DefaultStateType>& rubbishLocalTransitions, const R& n);

void localClosure(std::set<DefaultStateType>& states, const std::set<DefaultStateType>& oldStates, const automaton::RealTimeHeightDeterministicDPDA < > & d);
void localClosure(std::set<DefaultStateType>& states, const std::set<DefaultStateType>& oldStates, const automaton::VisiblyPushdownDPDA < > & d);

template<class T, class S, class R>
std::set<std::pair<DefaultStateType, DefaultSymbolType>> existsDirtyStateSymbol(const T& d, const std::set<DefaultStateType>& rubbishStates, const std::map<std::pair<DefaultStateType, S>, std::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions, const std::map<std::tuple<DefaultStateType, S, DefaultSymbolType>, DefaultStateType>& rubbishReturnTransitions,  const R& n);

} /* namespace determinize */

} /* namespace automaton */

#endif /* RHDPDA_VPA_COMMON_H_ */
