/*
 * MinimizeBrzozowski.cpp
 *
 *  Created on: 18. 11. 2014
 *	  Author: Tomas Pecka
 */
 
#ifndef MINIMIZE_BRZOZOWSKI_H_
#define MINIMIZE_BRZOZOWSKI_H_

#include <core/multipleDispatch.hpp>
#include <automaton/Automaton.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>

#include "../transform/Reverse.h"
#include "../determinize/Determinize.h"

namespace automaton {

namespace simplify {

class MinimizeBrzozowski : public std::SingleDispatch<MinimizeBrzozowski, automaton::Automaton, const automaton::AutomatonBase &> {
public:
	/**
	 * @param dfa automaton to minimize
	 */
	static automaton::Automaton minimize(const automaton::Automaton& dfa);

	template < class SymbolType, class StateType >
	static automaton::DFA < SymbolType, std::set < std::set < StateType > > > minimize(const automaton::DFA < SymbolType, StateType > & dfa);
	template < class SymbolType, class StateType >
	static automaton::DFA < SymbolType, std::set < std::set < StateType > > > minimize(const automaton::NFA < SymbolType, StateType > & nfa);
};

template < class SymbolType, class StateType >
automaton::DFA < SymbolType, std::set < std::set < StateType > > > MinimizeBrzozowski::minimize(const automaton::DFA < SymbolType, StateType > & dfa) {
	return automaton::determinize::Determinize::determinize(automaton::transform::Reverse::convert(automaton::determinize::Determinize::determinize(automaton::transform::Reverse::convert(dfa))));
}

template < class SymbolType, class StateType >
automaton::DFA < SymbolType, std::set < std::set < StateType > > > MinimizeBrzozowski::minimize(const automaton::NFA < SymbolType, StateType > & nfa) {
	return automaton::determinize::Determinize::determinize(automaton::transform::Reverse::convert(automaton::determinize::Determinize::determinize(automaton::transform::Reverse::convert(nfa))));
}

} /* namespace simplify */

} /* namespace automaton */

#endif /* MINIMIZE_BRZOZOWSKI_H_ */
