/*
 * Reverse.h
 *
 *  Created on: 7. 11. 2014
 *	  Author: Tomas Pecka
 */

#ifndef REVERSE_H_
#define REVERSE_H_

#include <vector>
#include <algorithm>

#include <core/multipleDispatch.hpp>

#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>
#include <automaton/FSM/MultiInitialStateNFA.h>

namespace automaton {

namespace transform {

class Reverse : public std::SingleDispatch<Reverse, automaton::Automaton, const automaton::AutomatonBase &> {
public:
	template < class SymbolType, class StateType >
	static automaton::MultiInitialStateNFA < SymbolType, StateType > convert(const automaton::DFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static automaton::MultiInitialStateNFA < SymbolType, StateType > convert(const automaton::NFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static automaton::MultiInitialStateNFA < SymbolType, StateType > convert(const automaton::MultiInitialStateNFA < SymbolType, StateType > & automaton);

	/**
	 * Creates finite automaton accepting reverse language of given automaton
	 * @param automaton FSM
	 * @return Automaton reverse automaton
	 */
	static automaton::Automaton convert( const automaton::Automaton & automaton );
};

template < class SymbolType, class StateType >
automaton::MultiInitialStateNFA < SymbolType, StateType > Reverse::convert(const automaton::DFA < SymbolType, StateType > & automaton) {
	automaton::MultiInitialStateNFA < SymbolType, StateType > res;

	res.setStates(automaton.getStates());
	res.addFinalState(automaton.getInitialState());
	res.setInitialStates( automaton.getFinalStates() );
	res.setInputAlphabet(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		res.addTransition(t.second, t.first.second, t.first.first);

	return res;
}

template < class SymbolType, class StateType >
automaton::MultiInitialStateNFA < SymbolType, StateType > Reverse::convert(const automaton::NFA < SymbolType, StateType > & automaton) {
	automaton::MultiInitialStateNFA < SymbolType, StateType > res;

	res.setStates(automaton.getStates());
	res.addFinalState(automaton.getInitialState());
	res.setInitialStates( automaton.getFinalStates() );
	res.setInputAlphabet(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		for(const auto& q : t.second)
			res.addTransition(q, t.first.second, t.first.first);

	return res;
}

template < class SymbolType, class StateType >
automaton::MultiInitialStateNFA < SymbolType, StateType > Reverse::convert(const automaton::MultiInitialStateNFA < SymbolType, StateType > & automaton) {
	automaton::MultiInitialStateNFA < SymbolType, StateType > res;

	res.setStates(automaton.getStates());
	res.setFinalStates(automaton.getInitialStates());
	res.setInitialStates( automaton.getFinalStates() );
	res.setInputAlphabet(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		for(const auto& q : t.second)
			res.addTransition(q, t.first.second, t.first.first);

	return res;
}

} /* namespace transform */

} /* namespace automaton */

#endif /* REVERSE_H_ */
