/*
 * Permutation.h
 *
 *  Created on: 10. 2. 2017
 *      Author: Jan Travnicek
 */

#ifndef PERMUTATION_H_
#define PERMUTATION_H_

#include <set>
#include <vector>
#include <map>
#include <random>
#include <algorithm>
#include <foreach>

#include <global/GlobalData.h>

namespace common {

class Permutation {
public:
	template < class T >
	static std::map < T, T > permutationMap ( const std::set < T > & data ) {
		std::vector < T > dataVector ( data.begin ( ), data.end ( ) );

		std::shuffle ( dataVector.begin ( ), dataVector.end ( ), std::random_devices::semirandom );
		std::map < T, T > permutation;
		for ( const std::tuple < const T &, const T & > & fromToPair : std::make_tuple_foreach ( data, dataVector ) ) {
			permutation.insert ( std::make_pair ( std::get < 0 > ( fromToPair ), std::get < 1 > ( fromToPair ) ) );
		}

		if(common::GlobalData::verbose)
			std::clog << "permutation map: " << permutation << std::endl;

		return permutation;
	}
};

} /* namespace common */

#endif /* PERMUTATION_H_ */
