/*
 * ToRegExp.h
 *
 *  Created on: 4. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef GRAMMAR_TO_REG_EXP_H_
#define GRAMMAR_TO_REG_EXP_H_

#include <core/multipleDispatch.hpp>

#include <grammar/Grammar.h>
#include <grammar/Regular/RightRG.h>
#include <grammar/Regular/LeftRG.h>

#include <regexp/RegExp.h>

#include "ToRegExpAlgebraic.h"

namespace grammar {

namespace convert {

class ToRegExp : public std::SingleDispatch<ToRegExp, regexp::RegExp, const grammar::GrammarBase &> {
public:
	/**
	 * @return regexp equivalent to source right regular grammar.
	 * @param grammar Grammar to convert
	 */
	static regexp::RegExp convert(const grammar::Grammar& grammar);

	template < class SymbolType >
	static regexp::RegExp convert(const grammar::RightRG < SymbolType > & grammar);
	template < class SymbolType >
	static regexp::RegExp convert(const grammar::LeftRG < SymbolType > & grammar);
};

template < class SymbolType >
regexp::RegExp ToRegExp::convert(const grammar::RightRG < SymbolType > & grammar) {
	return regexp::RegExp(ToRegExpAlgebraic::convert(grammar));
}

template < class SymbolType >
regexp::RegExp ToRegExp::convert(const grammar::LeftRG < SymbolType > & grammar) {
	return regexp::RegExp(ToRegExpAlgebraic::convert(grammar));
}

} /* namespace covert */

} /* namespace grammar */

#endif /* GRAMMAR_TO_REG_EXP_H_ */
