/*
 * CockeYoungerKasami.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef COCKE_YOUNGER_KASAMI_H_
#define COCKE_YOUNGER_KASAMI_H_

#include <core/multipleDispatch.hpp>
#include <global/GlobalData.h>

#include <grammar/Grammar.h>
#include <grammar/ContextFree/CNF.h>
#include <string/LinearString.h>

namespace grammar {

namespace generate {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class CockeYoungerKasami : public std::SingleDispatch < CockeYoungerKasami, bool, const grammar::GrammarBase &, const string::LinearString < > & > {
public:
	static bool generate ( const grammar::Grammar & grammar, const string::LinearString < > & string );

	template < class SymbolType >
	static bool generate ( const grammar::CNF < SymbolType > & grammar, const string::LinearString < SymbolType > & string );

};

template < class SymbolType >
bool CockeYoungerKasami::generate ( const grammar::CNF < SymbolType > & grammar, const string::LinearString < SymbolType > & string ) {
	unsigned stringSize = string.getContent ( ).size ( );

	if ( ( stringSize == 0 ) && grammar.getGeneratesEpsilon ( ) ) return true;

	std::vector < std::vector < std::set < SymbolType > > > data;
	data.resize ( stringSize );

	for ( unsigned i = 0; i < stringSize; i++ )
		data[i].resize ( stringSize - i );

	for ( unsigned i = 0; i < stringSize; i++ )
		for ( const std::pair < const SymbolType, std::set < std::variant < SymbolType, std::pair < SymbolType, SymbolType > > > > rule : grammar.getRules ( ) ) {
			const SymbolType & lhs = rule.first;

			for ( const std::variant < SymbolType, std::pair < SymbolType, SymbolType > > rhs : rule.second )
				if ( rhs.template is < SymbolType > ( ) && ( rhs.template get < SymbolType > ( ) == string.getContent ( )[i] ) )
					data[0][i].insert ( lhs );

		}

	for ( unsigned i = 1; i < stringSize; i++ )
		for ( unsigned j = 0; j < stringSize - i; j++ ) {
			std::set < SymbolType > & targetCell = data[i][j]; // Element to compute

			for ( unsigned k = 0; k < i; k++ ) {
				const std::set < SymbolType > & vertical = data[k][j];
				const std::set < SymbolType > & diagonal = data[i - 1 - k][j + 1 + k]; // Sources of data

				for ( const SymbolType & verticalElement : vertical ) {
					for ( const SymbolType & diagonalElement : diagonal )

						for ( const std::pair < const SymbolType, std::set < std::variant < SymbolType, std::pair < SymbolType, SymbolType > > > > rule : grammar.getRules ( ) ) {
							const SymbolType & lhs = rule.first;

							for ( const std::variant < SymbolType, std::pair < SymbolType, SymbolType > > rhs : rule.second )
								if ( rhs.template is < std::pair < SymbolType, SymbolType > > ( ) ) {
									const std::pair < SymbolType, SymbolType > rhsp = rhs.template get < std::pair < SymbolType, SymbolType > > ( );

									if ( ( rhsp.first == verticalElement ) && ( rhsp.second == diagonalElement ) )
										targetCell.insert ( lhs );
								}

						}

				}
			}
		}



	if ( common::GlobalData::verbose )
		for ( const std::vector < std::set < SymbolType > > & row : data ) {
			for ( const std::set < SymbolType > & element : row )
				std::clog << element << " ";

			std::clog << std::endl;
		}

	return data[stringSize - 1][0].count ( grammar.getInitialSymbol ( ) );
}

} /* namespace generate */

} /* namespace grammar */

#endif /* COCKE_YOUNGER_KASAMI_H_ */
