/*
 * RandomizeGrammar.h
 *
 *  Created on: 9. 2. 2017
 *	  Author: Jan Travnicek
 */

#ifndef GRAMMAR_RANDOMIZE_H_
#define GRAMMAR_RANDOMIZE_H_

#include <core/multipleDispatch.hpp>

#include <grammar/Grammar.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>
#include <grammar/ContextFree/CFG.h>

#include <common/Permutation.hpp>

namespace grammar {

namespace generate {

class RandomizeGrammar : public std::SingleDispatch < RandomizeGrammar, grammar::Grammar, const grammar::GrammarBase & > {
public:
	static grammar::Grammar randomize ( const grammar::Grammar & grammar );

	template < class SymbolType >
	static grammar::LeftRG < SymbolType > randomize ( const grammar::LeftRG < SymbolType > & gram );
	template < class SymbolType >
	static grammar::LeftLG < SymbolType > randomize ( const grammar::LeftLG < SymbolType > & gram );
	template < class SymbolType >
	static grammar::RightRG < SymbolType > randomize ( const grammar::RightRG < SymbolType > & gram );
	template < class SymbolType >
	static grammar::RightLG < SymbolType > randomize ( const grammar::RightLG < SymbolType > & gram );
	template < class SymbolType >
	static grammar::CFG < SymbolType > randomize ( const grammar::CFG < SymbolType > & gram );

};

template < class SymbolType >
grammar::LeftRG < SymbolType > RandomizeGrammar::randomize ( const grammar::LeftRG < SymbolType > & gram ) {
	std::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::LeftRG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, std::set < std::variant < SymbolType, std::pair < SymbolType, SymbolType > > > > & rule : gram.getRules ( ) )
		for ( const std::variant < SymbolType, std::pair < SymbolType, SymbolType > > & rhs : rule.second )
			if ( rhs.template is < SymbolType > ( ) )
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, rhs );
			else
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, std::make_pair ( symbolPermutationMap.find ( rhs.template get < std::pair < SymbolType, SymbolType > > ( ).first )->second, rhs.template get < std::pair < SymbolType, SymbolType > > ( ).second ) );

	return res;
}

template < class SymbolType >
grammar::LeftLG < SymbolType > RandomizeGrammar::randomize ( const grammar::LeftLG < SymbolType > & gram ) {
	std::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::LeftLG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, std::set < std::variant < std::vector < SymbolType >, std::pair < SymbolType, std::vector < SymbolType > > > > > & rule : gram.getRules ( ) )
		for ( const std::variant < std::vector < SymbolType >, std::pair < SymbolType, std::vector < SymbolType > > > & rhs : rule.second )
			if ( rhs.template is < std::vector < SymbolType > > ( ) )
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, rhs );
			else
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, std::make_pair ( symbolPermutationMap.find ( rhs.template get < std::pair < SymbolType, std::vector < SymbolType > > > ( ).first )->second, rhs.template get < std::pair < SymbolType, std::vector < SymbolType > > > ( ).second ) );

	return res;
}

template < class SymbolType >
grammar::RightRG < SymbolType > RandomizeGrammar::randomize ( const grammar::RightRG < SymbolType > & gram ) {
	std::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::RightRG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, std::set < std::variant < SymbolType, std::pair < SymbolType, SymbolType > > > > & rule : gram.getRules ( ) )
		for ( const std::variant < SymbolType, std::pair < SymbolType, SymbolType > > & rhs : rule.second )
			if ( rhs.template is < SymbolType > ( ) )
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, rhs );
			else
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, std::make_pair ( rhs.template get < std::pair < SymbolType, SymbolType > > ( ).first, symbolPermutationMap.find ( rhs.template get < std::pair < SymbolType, SymbolType > > ( ).second )->second ) );

	return res;
}

template < class SymbolType >
grammar::RightLG < SymbolType > RandomizeGrammar::randomize ( const grammar::RightLG < SymbolType > & gram ) {
	std::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::RightLG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, std::set < std::variant < std::vector < SymbolType >, std::pair < std::vector < SymbolType >, SymbolType > > > > & rule : gram.getRules ( ) )
		for ( const std::variant < std::vector < SymbolType >, std::pair < std::vector < SymbolType >, SymbolType > > & rhs : rule.second )
			if ( rhs.template is < std::vector < SymbolType > > ( ) )
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, rhs );
			else
				res.addRule ( symbolPermutationMap.find ( rule.first )->second, std::make_pair ( rhs.template get < std::pair < std::vector < SymbolType >, SymbolType > > ( ).first, symbolPermutationMap.find ( rhs.template get < std::pair < std::vector < SymbolType >, SymbolType > > ( ).second )->second ) );

	return res;
}

template < class SymbolType >
grammar::CFG < SymbolType > RandomizeGrammar::randomize ( const grammar::CFG < SymbolType > & gram ) {
	std::map < SymbolType, SymbolType > symbolPermutationMap = common::Permutation::permutationMap ( gram.getNonterminalAlphabet ( ) );

	grammar::CFG < SymbolType > res ( symbolPermutationMap.find ( gram.getInitialSymbol ( ) )->second );

	res.setNonterminalAlphabet ( gram.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( gram.getTerminalAlphabet ( ) );

	for ( const std::pair < SymbolType, std::set < std::vector < SymbolType > > > & rule : gram.getRules ( ) )
		for ( const std::vector < SymbolType > & rhs : rule.second )
			res.addRule ( symbolPermutationMap.find ( rule.first )->second, std::transform < SymbolType > ( rhs, [ & ] ( const SymbolType & elem ) {
						if ( gram.getNonterminalAlphabet ( ).count ( elem ) )
							return symbolPermutationMap.find ( elem )->second;
						else
							return elem;
					} ) );

	return res;
}

} /* namespace generate */

} /* namespace grammar */

#endif /* GRAMMAR_RANDOMIZE_H_ */
