/*
 * IsLanguageEmpty.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef IS_LANGUAGE_EMPTY_H_
#define IS_LANGUAGE_EMPTY_H_

#include <core/multipleDispatch.hpp>

#include <grammar/Grammar.h>

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <grammar/properties/ProductiveNonterminals.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class IsLanguageEmpty : public std::SingleDispatch<IsLanguageEmpty, bool, const grammar::GrammarBase &> {
public:
	static bool isLanguageEmpty( const grammar::Grammar & grammar );

	/*
	 * Melichar 3.6 - decides whether L( grammar ) = \0
	 *
	 * Severals steps implemented in method CFGTransformations::getProductiveNonTerminals();
	 * @see getProductiveNonTerminals
	 */
	template<class T>
	static bool isLanguageEmpty( const T & grammar );
};

template<class T>
bool IsLanguageEmpty::isLanguageEmpty( const T & grammar ) {
	return grammar::properties::ProductiveNonterminals::getProductiveNonterminals( grammar ).count( grammar.getInitialSymbol( ) );
}

} /* namespace properties */

} /* namespace grammar */

#endif /* IS_LANGUAGE_EMPTY_H_ */
