/*
 * RandomSubstringFactory.h
 *
 *  Created on: 27. 3. 2014
 *      Author: Jan Travnicek
 */

#ifndef RANDOM_SUBSTRING_FACTORY_H_
#define RANDOM_SUBSTRING_FACTORY_H_

#include <core/multipleDispatch.hpp>
#include <string/String.h>
#include <string/StringFeatures.h>

#include <algorithm>
#include <random>
#include <exception/CommonException.h>

#include <string/LinearString.h>

namespace string {

namespace generate {

class RandomSubstringFactory : public std::SingleDispatchFirstStaticParam < RandomSubstringFactory, string::String, size_t, const string::StringBase & > {
public:
	static string::String generateSubstring ( size_t size, const string::String & );

	template < class SymbolType >
	static string::LinearString < SymbolType > generateSubstring ( size_t size, const string::LinearString < SymbolType > & );

};

template < class SymbolType >
string::LinearString < SymbolType > RandomSubstringFactory::generateSubstring ( size_t size, const string::LinearString < SymbolType > & string ) {
	if ( size > string.getContent ( ).size ( ) )
		throw exception::CommonException ( "String not long enough" );

	size_t begin = std::random_devices::semirandom ( ) % ( string.getContent ( ).size ( ) - size + 1 );

	std::vector < SymbolType > data ( string.getContent ( ).begin ( ) + begin, string.getContent ( ).begin ( ) + begin + size );

	return LinearString < SymbolType > ( string.getAlphabet ( ), data );
}

} /* namespace generate */

} /* namespace string */

#endif /* RANDOM_SUBSTRING_FACTORY_H_ */
