/*
 * Author: Radovan Cerveny
 */

#ifndef NONDETERMINISTIC_EXACT_SUFFIX_AUTOMATON_H_
#define NONDETERMINISTIC_EXACT_SUFFIX_AUTOMATON_H_

#include <automaton/Automaton.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <string/LinearString.h>
#include <string/String.h>
#include <core/multipleDispatch.hpp>
#include <primitive/Unsigned.h>

namespace stringology {

namespace indexing {

class NondeterministicExactSuffixAutomaton : public std::SingleDispatch < NondeterministicExactSuffixAutomaton, automaton::Automaton, const string::StringBase & > {
public:
	/**
	 * Nondeterministic construction of nondeterministic suffix automaton for given pattern.
	 * @return nondeterministic suffix automaton for given pattern.
	 */
	static automaton::Automaton construct ( const string::String & pattern );

	static automaton::EpsilonNFA < DefaultSymbolType, DefaultEpsilonType, unsigned > construct ( const string::LinearString < > & pattern );

};

} /* namespace indexing */

} /* namespace stringology */

#endif /* NONDETERMINISTIC_EXACT_SUFFIX_AUTOMATON_H_ */
