/*
 * PositionHeapNaive.h
 *
 *  Created on: 6. 2. 2017
 *      Author: Jan Travnicek
 */

#ifndef POSITION_HEAP_NAIVE_H_
#define POSITION_HEAP_NAIVE_H_

#include <indexes/stringology/PositionHeap.h>
#include <string/String.h>
#include <string/LinearString.h>
#include <core/multipleDispatch.hpp>
#include <exception/CommonException.h>

namespace stringology {

namespace indexing {

/**
 * Constructs a position heap for given string.
 *
 * Source: Position heaps: A simple and dynamic text indexing data structure
 * Andrzej Ehrenfeucht, Ross M. McConnell, Nissa Osheim, Sung-Whan Woo
 */

class PositionHeapNaive : public std::SingleDispatch < PositionHeapNaive, indexes::stringology::PositionHeap < DefaultSymbolType >, const string::StringBase & > {
public:
	/**
	 * Creates suffix trie
	 * @param string string to construct suffix trie for
	 * @return automaton
	 */
	static indexes::stringology::PositionHeap < DefaultSymbolType > construct ( const string::String & string );

	template < class SymbolType >
	static indexes::stringology::PositionHeap < SymbolType > construct ( const string::LinearString < SymbolType > & string );

};

template < class SymbolType >
indexes::stringology::PositionHeap < SymbolType > PositionHeapNaive::construct ( const string::LinearString < SymbolType > & w ) {
	if ( w.getContent ( ).size ( ) == 0 )
		throw exception::CommonException ( "Position heap can't index empty string" );

	std::trie < SymbolType, unsigned > trie ( 1 );

	for ( unsigned i = w.getContent ( ).size ( ) - 1; i > 0; i-- ) {
		unsigned k = i - 1;
		std::trie < SymbolType, unsigned > * n = & trie;

		while ( n->getChildren ( ).count ( w.getContent ( )[k] ) )
			n = & n->getChildren ( ).find ( w.getContent ( )[k++] )->second;

		unsigned node = w.getContent ( ).size ( ) - i + 1;
		n = & n->getChildren ( ).insert ( std::make_pair ( w.getContent ( )[k], std::trie < SymbolType, unsigned > ( node ) ) ).first->second;
	}

	return indexes::stringology::PositionHeap < SymbolType > ( w.getAlphabet ( ), trie, w.getContent ( ) );
}

} /* namespace indexing */

} /* namespace stringology */

#endif /* POSITION_HEAP_NAIVE_H_ */
