/*
 * Author: Radovan Cerveny
 */

#ifndef NAIVE_DAWG_MATCHER_CONSTRUCTION_H_
#define NAIVE_DAWG_MATCHER_CONSTRUCTION_H_

#include <automaton/Automaton.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <string/LinearString.h>
#include <string/String.h>
#include <core/multipleDispatch.hpp>

namespace stringology {

namespace matching {

class NaiveDAWGMatcherConstruction : public std::SingleDispatch < NaiveDAWGMatcherConstruction, automaton::Automaton, const string::StringBase & > {
public:
	/**
	 * Naive construction of minimal suffix automaton for given pattern - EpsNFA -> NFA -> DFA -> minDFA -> removeErrorState.
	 * @return minimal suffix automaton for given pattern.
	 */
	static automaton::Automaton naiveConstruct ( const string::String & pattern );

	static automaton::DFA < DefaultSymbolType, std::set < unsigned > > naiveConstruct ( const string::LinearString < > & pattern );

};

} /* namespace matching */

} /* namespace stringology */

#endif /* NAIVE_DAWG_MATCHER_CONSTRUCTION_H_ */
