/*
 * Author: Radovan Cerveny
 */

#ifndef ORACLE_MATCHER_CONSTRUCTION_H_
#define ORACLE_MATCHER_CONSTRUCTION_H_

#include <automaton/Automaton.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <string/LinearString.h>
#include <string/String.h>
#include <core/multipleDispatch.hpp>

namespace stringology {

namespace matching {

class OracleMatcherConstruction : public std::SingleDispatch < OracleMatcherConstruction, automaton::Automaton, const string::StringBase & > {
public:
	 /**
	 * Linear time on-line construction of minimal suffix automaton for given pattern.
	 * @return minimal suffix automaton for given pattern.
	 */
	static automaton::Automaton construct ( const string::String & pattern );

	static automaton::DFA < > construct ( const string::LinearString < > & pattern );

};

} /* namespace matching */

} /* namespace stringology */

#endif /* ORACLE_MATCHER_CONSTRUCTION_H_ */
