/*
 * Author: Radovan Cerveny
 */

#include "BackwardOracleMatching.h"
#include "stringology/matching/OracleMatcherConstruction.h"
#include "stringology/properties/BackboneLength.h"

#include <string/LinearString.h>
#include <alphabet/Symbol.h>

#include <algorithm>
#include <map>
#include <measure>
#include <registration/AlgoRegistration.hpp>

namespace stringology {

namespace query {

std::set < unsigned > BackwardOracleMatching::match ( const string::String & subject, const automaton::Automaton & factorOracle ) {
	return dispatch ( subject.getData ( ), factorOracle.getData ( ) );
}

std::set < unsigned > BackwardOracleMatching::match ( const string::LinearString < > & subject, const automaton::DFA < > & factorOracle ) {
	std::set < unsigned > occ;

	size_t patternSize = stringology::properties::BackboneLength::length ( factorOracle );
	size_t subjectSize = subject.getContent ( ).size ( );

	const DefaultStateType failState = DefaultStateType ( -1 );

	size_t posInSubject = 0;

	while ( posInSubject <= subjectSize - patternSize ) {

		DefaultStateType currentState = factorOracle.getInitialState ( );

		size_t posInPattern = patternSize;

		while ( posInPattern > 0 && currentState != failState ) {
			auto transition = factorOracle.getTransitions ( ).find ( { currentState, subject.getContent ( ).at ( posInSubject + posInPattern - 1 ) } );

			if ( transition == factorOracle.getTransitions ( ).end ( ) )
				currentState = failState;
			else
				currentState = transition->second;

			posInPattern--;
		}

		if ( currentState != failState )
			 // Yay, there is match!!!
			occ.insert ( posInSubject );

		posInSubject += posInPattern + 1;
	}

	return occ;
}

auto BackwardOracleMatchingLinearStringLinearString = registration::OverloadRegister < BackwardOracleMatching, std::set < unsigned >, string::LinearString < >, automaton::DFA < > > ( BackwardOracleMatching::match );

} /* namespace query */

} /* namespace stringology */
