/*
 * ExactSubtreeRepeatsNaive.h
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#ifndef _ARBOLOGY_SUBTREE_REPEATS_NAIVE_H_
#define _ARBOLOGY_SUBTREE_REPEATS_NAIVE_H_

#include <tree/TreeFeatures.h>
#include <alphabet/SymbolFeatures.h>
#include <core/multipleDispatch.hpp>

#include <map>
#include <vector>
#include <tree>
#include <alphabet/RankedSymbol.h>
#include <primitive/Unsigned.h>

#include "SubtreeJumpTable.h"

#include <tree/ranked/RankedTree.h>
#include <tree/ranked/PrefixRankedTree.h>
#include <tree/ranked/PostfixRankedTree.h>
#include <tree/ranked/PrefixRankedBarTree.h>
#include <global/GlobalData.h>

namespace tree {

namespace properties {

/**
 * Simple computation of subtree repeats
 */
class ExactSubtreeRepeatsNaive : public std::SingleDispatch < ExactSubtreeRepeatsNaive, tree::Tree, const tree::TreeBase & > {
	template < class SymbolType, class RankType >
	static std::tree < std::ranked_symbol < unsigned, RankType > > repeats ( const std::tree < std::ranked_symbol < SymbolType, RankType > > & node, std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > & data, unsigned & minId );
	template < class SymbolType, class RankType >
	static std::ranked_symbol < unsigned, RankType > repeatsPrefixRanked ( const std::vector < std::ranked_symbol < SymbolType, RankType > > & symbols, std::vector < std::ranked_symbol < unsigned, RankType > > & res, std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > & data, unsigned & minId, int & index );
	template < class SymbolType, class RankType >
	static std::ranked_symbol < unsigned, RankType > repeatsPostfixRanked ( const std::vector < std::ranked_symbol < SymbolType, RankType > > & symbols, std::vector < std::ranked_symbol < unsigned, RankType > > & res, std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > & data, unsigned & minId, int & index );
	template < class SymbolType, class RankType >
	static std::ranked_symbol < unsigned, RankType > repeatsPrefixRankedBar ( const std::vector < std::ranked_symbol < SymbolType, RankType > > & symbols, std::vector < std::ranked_symbol < unsigned, RankType > > & res, std::map < std::pair < std::ranked_symbol <SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > & data, unsigned & minId, unsigned barId, int & index );

public:
	/**
	 * Compute a same shaped tree with nodes containing unique subtree ids.
	 * @return Tree of repeats
	 */
	static tree::Tree repeats ( const tree::Tree & pattern );

	/**
	 * Compute a same shaped tree with nodes containing unique subtree ids.
	 * @return Tree of repeats
	 */
	template < class SymbolType, class RankType >
	static tree::RankedTree < unsigned, RankType > repeats ( const tree::RankedTree < SymbolType, RankType > & tree );
	template < class SymbolType, class RankType >
	static tree::PrefixRankedTree < unsigned, RankType > repeats ( const tree::PrefixRankedTree < SymbolType, RankType > & tree );
	template < class SymbolType, class RankType >
	static tree::PostfixRankedTree < unsigned, RankType > repeats ( const tree::PostfixRankedTree < SymbolType, RankType > & tree );
	template < class SymbolType, class RankType >
	static tree::PrefixRankedBarTree < unsigned, RankType > repeats ( const tree::PrefixRankedBarTree < SymbolType, RankType > & tree );

};

template < class SymbolType, class RankType >
std::tree < std::ranked_symbol < unsigned, RankType > > ExactSubtreeRepeatsNaive::repeats ( const std::tree < std::ranked_symbol < SymbolType, RankType > > & node, std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > & data, unsigned & minId ) {
	std::vector < std::tree < std::ranked_symbol < unsigned, RankType > > > children;
	std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > > childRepeatsKey ( node.getData ( ), std::vector < std::ranked_symbol < unsigned, RankType > > ( ) );

	for ( const std::tree < std::ranked_symbol < SymbolType, RankType > > & child : node.getChildren() ) {
		children.push_back ( repeats ( child, data, minId ) );
		childRepeatsKey.second.push_back ( children.back ( ).getData ( ) );
	}

	unsigned & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	return std::tree < std::ranked_symbol < unsigned, RankType > > ( std::ranked_symbol < unsigned, RankType > ( uniqueRepeatId, node.getData ( ).getRank ( ) ), std::move ( children ) );
}

template < class SymbolType, class RankType >
tree::RankedTree < unsigned, RankType > ExactSubtreeRepeatsNaive::repeats ( const tree::RankedTree < SymbolType, RankType > & tree ) {
	unsigned minId = 1;
	std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > data;

	return tree::RankedTree < unsigned, RankType > ( repeats ( tree.getContent ( ), data, minId ) );
}

template < class SymbolType, class RankType >
std::ranked_symbol < unsigned, RankType > ExactSubtreeRepeatsNaive::repeatsPrefixRanked ( const std::vector < std::ranked_symbol < SymbolType, RankType > > & symbols, std::vector < std::ranked_symbol < unsigned, RankType > > & res, std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > & data, unsigned & minId, int & index ) {
	int begin = index;
	std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > > childRepeatsKey ( symbols[begin], std::vector < std::ranked_symbol < unsigned, RankType > > ( ) );

	res.push_back ( std::ranked_symbol < unsigned, RankType > ( 0, symbols[begin].getRank ( ) ) );

	index++;

	for ( unsigned i = 0; i < ( unsigned ) symbols[begin].getRank ( ); ++i )
		childRepeatsKey.second.push_back ( repeatsPrefixRanked ( symbols, res, data, minId, index ) );

	unsigned & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	res[begin] = std::ranked_symbol < unsigned, RankType > ( uniqueRepeatId, symbols[begin].getRank ( ) );
	return res[begin];
}

template < class SymbolType, class RankType >
tree::PrefixRankedTree < unsigned, RankType > ExactSubtreeRepeatsNaive::repeats ( const tree::PrefixRankedTree < SymbolType, RankType > & tree ) {
	unsigned minId = 1;
	int index = 0;
	std::vector < std::ranked_symbol < unsigned, RankType > > res;
	std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > data;

	repeatsPrefixRanked ( tree.getContent ( ), res, data, minId, index );
	return tree::PrefixRankedTree < unsigned, RankType > ( res );
}

template < class SymbolType, class RankType >
std::ranked_symbol < unsigned, RankType > ExactSubtreeRepeatsNaive::repeatsPostfixRanked ( const std::vector < std::ranked_symbol < SymbolType, RankType > > & symbols, std::vector < std::ranked_symbol < unsigned, RankType > > & res, std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > & data, unsigned & minId, int & index ) {
	int begin = index;

	std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > > childRepeatsKey ( symbols[begin], std::vector < std::ranked_symbol < unsigned, RankType > > ( ) );
	res.insert ( res.begin ( ), std::ranked_symbol < unsigned, RankType > ( 0, symbols[begin].getRank ( ) ) );

	index--;

	for ( unsigned i = symbols.size ( ) - 1; i > symbols.size ( ) - 1 - ( unsigned ) symbols[begin].getRank ( ); --i )
		childRepeatsKey.second.push_back ( repeatsPostfixRanked ( symbols, res, data, minId, index ) );

	unsigned & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	unsigned pos_in_res = begin - symbols.size ( ) + res.size ( );
	res[pos_in_res] = std::ranked_symbol < unsigned, RankType > ( uniqueRepeatId, symbols[begin].getRank ( ) );
	return res[pos_in_res];
}

template < class SymbolType, class RankType >
tree::PostfixRankedTree < unsigned, RankType > ExactSubtreeRepeatsNaive::repeats ( const tree::PostfixRankedTree < SymbolType, RankType > & tree ) {
	unsigned minId = 1;
	int index = tree.getContent ( ).size ( ) - 1;

	std::vector < std::ranked_symbol < unsigned, RankType > > res;
	std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > data;

	repeatsPostfixRanked ( tree.getContent ( ), res, data, minId, index );
	return tree::PostfixRankedTree < unsigned, RankType > ( res );
}


template < class SymbolType, class RankType >
std::ranked_symbol < unsigned, RankType > ExactSubtreeRepeatsNaive::repeatsPrefixRankedBar ( const std::vector < std::ranked_symbol < SymbolType, RankType > > & symbols, std::vector < std::ranked_symbol < unsigned, RankType > > & res, std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > & data, unsigned & minId, unsigned barId, int & index ) {
	int begin = index;
	std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > > childRepeatsKey ( symbols[begin], std::vector < std::ranked_symbol < unsigned, RankType > > ( ) );

	res.push_back ( std::ranked_symbol < unsigned, RankType > ( 0, symbols[begin].getRank ( ) ) );

	index++;

	for ( unsigned i = 0; i < ( unsigned ) symbols[begin].getRank ( ); ++i )
		childRepeatsKey.second.push_back ( repeatsPrefixRankedBar ( symbols, res, data, minId, barId, index ) );

	unsigned & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	res[begin] = std::ranked_symbol < unsigned, RankType > ( uniqueRepeatId, symbols[begin].getRank ( ) );
	res.push_back ( std::ranked_symbol < unsigned, RankType > ( 0, symbols[index].getRank ( ) ) );
	index++;

	return res[begin];
}

template < class SymbolType, class RankType >
tree::PrefixRankedBarTree < unsigned, RankType > ExactSubtreeRepeatsNaive::repeats ( const tree::PrefixRankedBarTree < SymbolType, RankType > & tree ) {
	unsigned barId = 0;
	unsigned minId = barId + 1;
	int index = 0;
	std::vector < std::ranked_symbol < unsigned, RankType > > res;
	std::map < std::pair < std::ranked_symbol < SymbolType, RankType >, std::vector < std::ranked_symbol < unsigned, RankType > > >, unsigned > data;

	repeatsPrefixRankedBar ( tree.getContent ( ), res, data, minId, barId, index );

	std::set < std::ranked_symbol < unsigned, RankType > > bars;
	for ( const std::ranked_symbol < SymbolType, RankType > & bar : tree.getBars ( ) )
		bars.insert ( std::ranked_symbol < unsigned, RankType > ( barId, bar.getRank ( ) ) );

	return tree::PrefixRankedBarTree < unsigned, RankType > ( bars, res );
}

} /* namespace properties */

} /* namespace tree */

#endif /* _ARBOLOGY_SUBTREE_REPEATS_NAIVE_H_ */
