/*
 * Dfs.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#include "Dfs.h"

#include <stack>
#include <unordered_map>
#include <registration/AlgoRegistration.hpp>

namespace graph {

namespace traverse {

template <typename T>
static void dfs_impl(const T &graph, const Node &start, Dfs::Function func) {
	std::unordered_map<Node, bool> visited;
	std::stack<Node> s;

	s.push(start);
	visited[start] = true;

	while (!s.empty()) {
		Node n = s.top(); s.pop();

		if (!func(n)) {
			return;
		}

		for (const Node &e : graph.neighbors(n)) {
			if (visited.find(e) == visited.end()) {
				visited[e] = true;
				s.push(e);
			}
		}
	}
}

template <typename T>
static bool dfs2_impl(const T &graph, const Node &n, const Node &p, std::unordered_map<Node, bool> &visited, int &time, Dfs::FunctionExt func) {
	int opened = ++time;
	visited[n] = true;

	for (const Node &e : graph.neighbors(n)) {
		if (visited.find(e) == visited.end()) {
			if (!dfs2_impl(graph, e, n, visited, time, func)) {
				return false;
			}
		}
	}

	return func(n, p, opened, ++time);
}

void Dfs::dfs(const Graph &graph, const Node &start, Dfs::Function func) {
	dispatch(graph.getData(), start, func);
}

void Dfs::dfs(const Graph &graph, const Node &start, Dfs::FunctionExt func) {
	dispatch(graph.getData(), start, func);
}

void Dfs::dfs(const DirectedGraph &graph, const Node &start, Dfs::Function func) {
	dfs_impl(graph, start, func);
}

auto DfsDirectedGraph1 = registration::OverloadRegister < std::SingleDispatch<Dfs, void, const graph::GraphBase &, const Node&, Dfs::Function>, void, graph::DirectedGraph> ( Dfs::dfs );

void Dfs::dfs(const DirectedGraph &graph, const Node &start, Dfs::FunctionExt func) {
	int time = 0;
	std::unordered_map<Node, bool> visited;
	dfs2_impl(graph, start, start, visited, time, func);
}

auto DfsDirectedGraph2 = registration::OverloadRegister < std::SingleDispatch<Dfs, void, const graph::GraphBase &, const Node&, Dfs::FunctionExt>, void, graph::DirectedGraph> ( Dfs::dfs );

void Dfs::dfs(const UndirectedGraph &graph, const Node &start, Dfs::Function func) {
	dfs_impl(graph, start, func);
}

auto DfsUndirectedGraph1 = registration::OverloadRegister < std::SingleDispatch<Dfs, void, const graph::GraphBase &, const Node&, Dfs::Function>, void, graph::UndirectedGraph> ( Dfs::dfs );

void Dfs::dfs(const UndirectedGraph &graph, const Node &start, Dfs::FunctionExt func) {
	int time = 0;
	std::unordered_map<Node, bool> visited;
	dfs2_impl(graph, start, start, visited, time, func);
}

auto DfsUndirectedGraph2 = registration::OverloadRegister < std::SingleDispatch<Dfs, void, const graph::GraphBase &, const Node&, Dfs::FunctionExt>, void, graph::UndirectedGraph> ( Dfs::dfs );

} // namespace traverse

} // namespace graph
