/*
 * Dfs.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef GRAPH_DFS_H_
#define GRAPH_DFS_H_

#include <core/multipleDispatch.hpp>
#include <graph/GraphClasses.h>

namespace graph {

namespace traverse {

// func is called for each visited node, traversal stops when returning false
//
// bool Function(const Node &node);
// bool FunctionExt(const Node &node, const Node &predecessor, int openTime, int closeTime);

class Dfs : public std::SingleDispatch<Dfs, void, const graph::GraphBase &, const Node&, std::function<bool(const Node&)>>, public std::SingleDispatch<Dfs, void, const graph::GraphBase &, const Node&, std::function<bool(const Node&, const Node&, int, int)>> {
public:
	using std::SingleDispatch<Dfs, void, const graph::GraphBase &, const Node&, std::function<bool(const Node&)>>::dispatch;
	using std::SingleDispatch<Dfs, void, const graph::GraphBase &, const Node&, std::function<bool(const Node&, const Node&, int, int)>>::dispatch;

	typedef std::function<bool(const Node&)> Function;
	typedef std::function<bool(const Node&, const Node&, int, int)> FunctionExt;

	static void dfs(const Graph &graph, const Node &start, Function func);
	static void dfs(const Graph &graph, const Node &start, FunctionExt func);

	static void dfs(const DirectedGraph &graph, const Node &start, Function func);
	static void dfs(const DirectedGraph &graph, const Node &start, FunctionExt func);
	static void dfs(const UndirectedGraph &graph, const Node &start, Function func);
	static void dfs(const UndirectedGraph &graph, const Node &start, FunctionExt func);
};

} // namespace traverse

} // namespace graph

#endif // GRAPH_DFS_H_
