#ifndef _CLI_SINGLE_STATEMENT_H_
#define _CLI_SINGLE_STATEMENT_H_

#include <ast/Statement.h>
#include <abstraction/Registry.h>
#include <exception/CommonException.h>

namespace cli {

class SingleStatement : public Statement {
	std::string m_name;
	std::vector < std::unique_ptr < Param > > m_params;

public:
	SingleStatement ( std::string name, std::vector < std::unique_ptr < Param > > params ) : m_name ( name ), m_params ( std::move ( params ) ) {
	}

	virtual std::shared_ptr < abstraction::OperationAbstraction > translateAndEval ( const std::shared_ptr < abstraction::OperationAbstraction > & prev, Environment & environment ) const override {
		std::vector < std::shared_ptr < abstraction::OperationAbstraction > > params;
		for ( const std::unique_ptr < Param > & param : m_params ) {
			params.push_back ( param->translateAndEval ( prev, environment ) );
		}

		std::vector < std::type_index > paramTypes;
		for ( const  std::shared_ptr < abstraction::OperationAbstraction > & param : params ) {
			paramTypes.push_back ( param->type ( ) );
		}

		bool downcast = false;
		bool normalize = false;

		std::shared_ptr < abstraction::OperationAbstraction > algo = abstraction::Registry::getAlgorithmAbstraction ( m_name, paramTypes, downcast, normalize );

		unsigned i = 0;
		for ( const std::shared_ptr < abstraction::OperationAbstraction > & param : params ) {
			if ( ! algo->attachInput ( param, i ) )
				throw exception::CommonException ( "Can't connect param at " + std::to_string ( i ) + " of algorithm " + m_name + " with result of type " + std::type_name ( param->type ( ) ) + "." );
			i++;
		}

		if ( ! algo->eval ( ) )
			throw exception::CommonException ( "Eval of algorithm " + m_name + " failed." );

		if ( downcast ) {
			std::shared_ptr < abstraction::OperationAbstraction > downcaster = algo->getDowncastResult ( );
			downcaster->attachInput ( algo, 0 );
			downcaster->eval ( );

			algo = downcaster;
		}

		if ( normalize ) {
			std::shared_ptr < abstraction::OperationAbstraction > normalized = algo->getNormalizeResult ( );
			normalized->attachInput ( algo, 0 );
			normalized->eval ( );

			algo = normalized;
		}

		return algo;
	}

};

} /* namespace cli */

#endif /* _CLI_SINGLE_STATEMENT_H_ */
