#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_

#include <string>
#include <map>

namespace cli {

class Environment {
	std::map < std::string, std::string > m_bindings;

	std::unique_ptr < Environment > m_upper;
public:
	Environment ( ) {
	}

	Environment ( std::unique_ptr < Environment > upper ) : m_upper ( std::move ( upper ) ) {
	}

	std::string get ( const std::string & name ) const {
		auto res = m_bindings.find ( name );

		if ( res != m_bindings.end ( ) )
			return res->second;

		if ( ( bool ) m_upper )
			return m_upper->get ( name );
		else
			throw exception::CommonException ( "Binded value not found for " + name + "." );
	}

	bool set ( std::string name, std::string value ) {
		return m_bindings.insert ( std::make_pair ( std::move ( name ), std::move ( value ) ) ).second;
	}

	bool clear ( const std::string & name ) {
		return m_bindings.erase ( name );
	}
};

} /* namespace cli */

#endif /* _ENVIRONMENT_H_ */
