#ifndef __CLI_LEXER_H_
#define __CLI_LEXER_H_

#include <string>
#include <iostream>

namespace cli {

class Lexer {
	unsigned m_index;
	std::string m_line;

public:
	enum class TokenType {
		IDENTIFIER,
		INTEGER,
		IN_REDIRECT,
		OUT_REDIRECT,
		LEFT_PAREN,
		RIGHT_PAREN,
		DOLAR_SIGN,
		AMPERSAND_SIGN,
		PIPE_SIGN,
		COLON_SIGN,
		DASH_SIGN,
		ERROR,
		END
	};

	static TokenType is_kw ( const std::string & /* identifier */ ) {
		return TokenType::IDENTIFIER;
	}

	struct Token {
		std::string m_value;
		TokenType m_type;

		friend std::ostream & operator << ( std::ostream & out, const Token & token ) {
			switch ( token.m_type ) {
			case TokenType::IDENTIFIER :
				return out << "identifier: " << token.m_value;
			case TokenType::INTEGER :
				return out << "number: " << token.m_value;
			case TokenType::IN_REDIRECT :
				return out << "in_redirect";
			case TokenType::OUT_REDIRECT :
				return out << "out_redirect";
			case TokenType::LEFT_PAREN :
				return out << "left_paren";
			case TokenType::RIGHT_PAREN :
				return out << "right_paren";
			case TokenType::DOLAR_SIGN :
				return out << "dolar_sign";
			case TokenType::AMPERSAND_SIGN :
				return out << "ampersand_sign";
			case TokenType::PIPE_SIGN :
				return out << "pipe_sign";
			case TokenType::COLON_SIGN :
				return out << "colon_sign";
			case TokenType::DASH_SIGN :
				return out << "dash_sign";
			case TokenType::ERROR :
				return out << "error";
			case TokenType::END :
				return out << "end";
			}
		}
	};

	Lexer ( std::string line ) : m_index ( 0 ), m_line ( std::move ( line ) ) {
	}

	Token nextToken ( );
};

} /* cli */

#endif /* _CLI_LEXER_H_ */
