/*
 * NullaryOperationAbstraction.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _NULLARY_OPERATION_ABSTRACTION_HPP_
#define _NULLARY_OPERATION_ABSTRACTION_HPP_

#include <abstraction/ResultClassSelector.hpp>
#include <exception/CommonException.h>

namespace abstraction {

template < class ReturnType >
class NullaryOperationAbstraction : public ResultClassSelector < ReturnType >::OutputClass {
	virtual bool attachInput ( const std::shared_ptr < OperationAbstraction > &, unsigned ) override {
		return false;
	}

	virtual bool detachInput ( const std::shared_ptr < OperationAbstraction > & ) override {
		return true;
	}

public:
	NullaryOperationAbstraction ( ) {
	}

	virtual bool inputsReady ( ) const override {
		return true;
	}

	virtual bool inputsAttached ( ) const override {
		return true;
	}

	virtual bool eval ( ) override {
		if ( this->cached ( ) )
			return true;

		return this->run ( );
	}

	virtual unsigned numberOfParams ( ) const override {
		return 0;
	}

	virtual std::shared_ptr < OperationAbstraction > getImmediateValueFromParam ( unsigned, const std::string & ) const override {
		throw exception::CommonException ( "Nullary algorithm does not have any parameter." );
	}

	virtual std::shared_ptr < OperationAbstraction > getXmlParserFromParam ( unsigned, std::deque < sax::Token > ) const override {
		throw exception::CommonException ( "Nullary algorithm does not have any parameter." );
	}
};

} /* namespace abstraction */

#endif /* _NULLARY_OPERATION_ABSTRACTION_HPP_ */
