/*
 * OperationAbstraction.hpp
 *
 *  Created on: 10. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _OPERATION_ABSTRACTION_HPP_
#define _OPERATION_ABSTRACTION_HPP_

#include <memory>
#include <string>
#include <tuple>
#include <functional>

#include <exception/CommonException.h>
#include <factory/XmlDataFactory.hpp>

namespace abstraction {

class OperationAbstraction : public std::enable_shared_from_this < OperationAbstraction > {
public:
	virtual bool attachInput ( const std::shared_ptr < OperationAbstraction > & input, unsigned index ) = 0;
	virtual bool detachInput ( const std::shared_ptr < OperationAbstraction > & input ) = 0;

	virtual ~OperationAbstraction ( ) noexcept {
	}

	virtual bool inputsReady ( ) const = 0;
	virtual bool inputsAttached ( ) const = 0;
	virtual bool run ( ) = 0;
	virtual bool eval ( ) = 0;
	virtual std::type_index type ( ) const = 0;
	virtual std::type_index runtime_type ( ) const = 0;
	virtual unsigned numberOfParams ( ) const = 0;
	virtual bool cached ( ) const = 0;

	virtual std::shared_ptr < OperationAbstraction > getXmlParserFromParam ( unsigned index, std::deque < sax::Token > tokens ) const = 0;
	virtual std::shared_ptr < OperationAbstraction > getImmediateValueFromParam ( unsigned index, const std::string & data ) const = 0;

	virtual std::shared_ptr < OperationAbstraction > getXmlFileWriterFromResult ( const std::string & filename ) const = 0;
	virtual std::shared_ptr < OperationAbstraction > getValuePrinterFromResult ( ) const = 0;
	virtual std::shared_ptr < OperationAbstraction > getCastFromResult ( const std::string & type ) const = 0;
	virtual std::shared_ptr < OperationAbstraction > getNormalizeResult ( ) const = 0;
	virtual std::shared_ptr < OperationAbstraction > getDowncastResult ( ) const = 0;

};

} /* namespace abstraction */

#endif /* _OPERATION_ABSTRACTION_HPP_ */
