/*
 * PrimitiveRegistrator.cpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#include "CastRegistry.hpp"
#include "XmlFileWriterRegistry.hpp"
#include "NormalizeRegistry.hpp"
#include "ValuePrinterRegistry.hpp"
#include "ImmediateRegistry.hpp"

#include <primitive/Double.h>
#include <primitive/Integer.h>
#include <primitive/Bool.h>
#include <primitive/UnsignedLong.h>
#include <primitive/String.h>

namespace abstraction {

class PrimitiveCasts {
public:
	PrimitiveCasts ( ) {
		abstraction::CastRegistry::registerCast < double, int > ( );
		abstraction::CastRegistry::registerCast < int, double > ( );

		abstraction::CastRegistry::registerCastAlgorithm < std::string, int > ( std::to_string );
		abstraction::CastRegistry::registerCastAlgorithm < int, std::string > ( (int(*)(std::string)) std::from_string < int > );

		abstraction::CastRegistry::registerCast < bool, int > ( );

		abstraction::CastRegistry::registerCast < size_t, int > ( "size_t", std::type_name < int > ( ) );
		abstraction::CastRegistry::registerCast < size_t, int > ( );
		abstraction::CastRegistry::registerCast < int, size_t > ( );

		abstraction::CastRegistry::registerCast < int, primitive::Integer > ( );
	
		abstraction::XmlFileWriterRegistry::registerXmlFileWriter < int > ( );
		abstraction::XmlFileWriterRegistry::registerXmlFileWriter < double > ( );
		abstraction::XmlFileWriterRegistry::registerXmlFileWriter < std::string > ( );

		abstraction::ValuePrinterRegistry::registerValuePrinter < int > ( );
		abstraction::ValuePrinterRegistry::registerValuePrinter < double > ( );
		abstraction::ValuePrinterRegistry::registerValuePrinter < std::string > ( );

		abstraction::ImmediateRegistry::registerImmediate < int > ( );
		abstraction::ImmediateRegistry::registerImmediate < std::string > ( );
	}
};

auto primitiveCasts = PrimitiveCasts ( );

} /* namespace abstraction */
