/*
 * Registry.cpp
 *
 *  Created on: 10. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <abstraction/Registry.h>

#include <abstraction/AlgorithmRegistry.hpp>
#include <abstraction/ImmediateRegistry.hpp>
#include <abstraction/XmlParserRegistry.hpp>
#include <abstraction/XmlFileWriterRegistry.hpp>
#include <abstraction/ValuePrinterRegistry.hpp>
#include <abstraction/CastRegistry.hpp>
#include <abstraction/NormalizeRegistry.hpp>
#include <abstraction/DowncastRegistry.hpp>

namespace abstraction {

std::shared_ptr < abstraction::OperationAbstraction > Registry::getAlgorithmAbstraction ( std::string name, const std::vector < std::type_index > & paramTypes ) {
	return AlgorithmRegistry::getAbstraction ( std::move ( name ), paramTypes );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getAlgorithmAbstraction ( std::string name, const std::vector < std::type_index > & paramTypes, bool & unwrap, bool & normalize ) {
	return AlgorithmRegistry::getAbstraction ( std::move ( name ), paramTypes, unwrap, normalize );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getCastAbstraction ( std::string target, std::string param ) {
	return CastRegistry::getAbstraction ( std::move ( target ), std::move ( param ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getImmediateAbstraction ( std::string result, std::string value ) {
	return ImmediateRegistry::getAbstraction ( std::move ( result ), std::move ( value ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getNormalizeAbstraction ( std::string param ) {
	return NormalizeRegistry::getAbstraction ( std::move ( param ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getDowncastAbstraction ( std::string concrete, std::string base ) {
	return DowncastRegistry::getAbstraction ( std::move ( concrete ), std::move ( base ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getValuePrinterAbstraction ( std::string param ) {
	return ValuePrinterRegistry::getAbstraction ( std::move ( param ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getXmlFileWriterAbstraction ( std::string param, std::string filename ) {
	return XmlFileWriterRegistry::getAbstraction ( std::move ( param ), std::move ( filename ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getXmlParserAbstraction ( std::string typeName, std::deque < sax::Token > tokens ) {
	return XmlParserRegistry::getAbstraction ( std::move ( typeName ), std::move ( tokens ) );
}

} /* namespace abstraction */
