/*
 * ValuePrinterRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _VALUE_PRINTER_REGISTRY_HPP_
#define _VALUE_PRINTER_REGISTRY_HPP_

#include <memory>
#include <string>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class ValuePrinterRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const = 0;

	};

	template < class Param >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const override;
	};

	static std::map < std::string, std::unique_ptr < Entry > > & getEntries ( ) {
		static std::map < std::string, std::unique_ptr < Entry > > fileWriters;
		return fileWriters;
	}

public:
	template < class ParamType >
	static void registerValuePrinter ( std::string param ) {
		getEntries ( ).insert ( std::make_pair ( param, std::unique_ptr < Entry > ( new EntryImpl < ParamType > ( ) ) ) );
	}

	template < class ParamType >
	static void registerValuePrinter ( ) {
		std::string param = std::type_name < ParamType > ( );
		registerValuePrinter < ParamType > ( std::move ( param ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::string param ) {
		auto res = getEntries ( ).find ( param );
		if ( res == getEntries ( ).end ( ) )
			throw exception::CommonException ( "Entry " + param + " not available." );

		return res->second->getAbstraction ( );
	}
};

} /* namespace abstraction */

#include <abstraction/ValuePrinterAbstraction.hpp>

namespace abstraction {

template < class Param >
std::shared_ptr < abstraction::OperationAbstraction > ValuePrinterRegistry::EntryImpl < Param >::getAbstraction ( ) const {
	return std::make_shared < abstraction::ValuePrinterAbstraction < Param > > ( );
}

} /* namespace abstraction */

#endif /* _VALUE_PRINTER_REGISTRY_HPP_ */
