/*
 * Map.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_VARIANT_H_
#define OBJECTS_VARIANT_H_

#include <variant>
#include <string>

#include <core/xmlApi.hpp>
#include <exception/CommonException.h>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsVariant {
private:
	explicit ObjectsVariant ( );

public:
	template < class ... Ts >
	static std::variant < Ts ... > parseRaw ( std::deque < sax::Token >::iterator & input );

	template < class ... Ts >
	static void compose ( std::deque < sax::Token > & out, const std::variant < Ts ... > & input );
};

template < typename T >
T parseVariantHelper ( std::deque < sax::Token >::iterator & ) {
	throw exception::CommonException ( "Can't parse variant type" );
}

template < typename T, typename R, typename ... Ts, typename std::enable_if < ! std::is_same < R, void >::value >::type * = nullptr >
T parseVariantHelper ( std::deque < sax::Token >::iterator & input );

template < typename T, typename R, typename ... Ts, typename std::enable_if < std::is_same < R, void >::value >::type * = nullptr >
T parseVariantHelper ( std::deque < sax::Token >::iterator & input ) {
	if ( alib::xmlApi < R >::first ( input ) ) {
		alib::xmlApi < R >::parse ( input );
		return T::template from < R > ( );
	}

	return parseVariantHelper < T, Ts ... > ( input );
}

template < typename T, typename R, typename ... Ts, typename std::enable_if < ! std::is_same < R, void >::value >::type * >
T parseVariantHelper ( std::deque < sax::Token >::iterator & input ) {
	if ( alib::xmlApi < R >::first ( input ) ) return T ( alib::xmlApi < R >::parse ( input ) );

	return parseVariantHelper < T, Ts ... > ( input );
}

template < typename ... Ts >
std::variant < Ts ... > ObjectsVariant::parseRaw ( std::deque < sax::Token >::iterator & input ) {
	return parseVariantHelper < std::variant < Ts ... >, Ts ... > ( input );
}

template < typename T >
bool firstVariantHelper ( std::deque < sax::Token >::iterator & ) {
	return false;
}

template < typename T, typename R, typename ... Ts >
T firstVariantHelper ( std::deque < sax::Token >::iterator & input ) {
	if ( alib::xmlApi < R >::first ( input ) ) return true;

	return firstVariantHelper < T, Ts ... > ( input );
}

template < typename T >
void composeVariantHelper ( std::deque < sax::Token > &, const T & ) {
	throw exception::CommonException ( "Can't compose variant type" );
}

template < typename T, typename R, typename ... Ts, typename std::enable_if < ! std::is_same < R, void >::value >::type * = nullptr >
void composeVariantHelper ( std::deque < sax::Token > & out, const T & container );

template < typename T, typename R, typename ... Ts, typename std::enable_if < std::is_same < R, void >::value >::type * = nullptr >
void composeVariantHelper ( std::deque < sax::Token > & out, const T & container ) {
	if ( container.template is < R > ( ) )
		return alib::xmlApi < R >::compose ( out );

	composeVariantHelper < T, Ts ... > ( out, container );
}

template < typename T, typename R, typename ... Ts, typename std::enable_if < ! std::is_same < R, void >::value >::type * >
void composeVariantHelper ( std::deque < sax::Token > & out, const T & container ) {
	if ( container.template is < R > ( ) ) 
		return alib::xmlApi < R >::compose ( out, container.template get < R > ( ) );

	composeVariantHelper < T, Ts ... > ( out, container );
}

template < typename ... Ts >
void ObjectsVariant::compose ( std::deque < sax::Token > & out, const std::variant < Ts ... > & container ) {
	composeVariantHelper < std::variant < Ts ... >, Ts ... > ( out, container );
}

} /* namespace container */

namespace alib {

template < typename ... Ts >
struct xmlApi < std::variant < Ts ... > > {
	static std::variant < Ts ... > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( std::deque < sax::Token > & output, const std::variant < Ts ... > & data );
};

template < typename ... Ts >
std::variant < Ts ... > xmlApi < std::variant < Ts ... > >::parse ( std::deque < sax::Token >::iterator & input ) {
	return container::ObjectsVariant::parseRaw < Ts ... > ( input );
}

template < typename ... Ts >
bool xmlApi < std::variant < Ts ... > >::first ( const std::deque < sax::Token >::const_iterator & input ) {
	return container::firstVariantHelper < std::variant < Ts ... >, Ts ... > ( input );
}

template < typename ... Ts >
std::string xmlApi < std::variant < Ts ... > >::xmlTagName ( ) {
	throw ::exception::CommonException ( "Variant does not have xmlTagName." );
}

template < typename ... Ts >
void xmlApi < std::variant < Ts ... > >::compose ( std::deque < sax::Token > & output, const std::variant < Ts ... > & input ) {
	return container::ObjectsVariant::compose < Ts ... > ( output, input );
}

} /* namespace alib */

#endif /* OBJECTS_VARIANT_H_ */
