/*
 * Void.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Void.h"
#include "../sax/FromXMLParserHelper.h"
#include "Object.h"
#include "UniqueObject.h"

#include <registration/TypeRegistration.hpp>

namespace alib {

Void::Void() {

}

ObjectBase* Void::clone() const {
	return new Void(*this);
}

ObjectBase* Void::plunder() && {
	return new Void(std::move(*this));
}

int Void::compare(const Void&) const {
	return 0;
}

void Void::operator>>(std::ostream& out) const {
	out << "(Void)";
}

Void::operator std::string () const {
	return "V";
}

Void Void::VOID = Void();

Void Void::parse(std::deque<sax::Token>::iterator& input) {
	xmlApi < void >::parse ( input );
	return Void();
}

void Void::compose(std::deque<sax::Token>& out) const {
	xmlApi < void >::compose ( out );
}

ObjectBase* Void::inc() && {
	return new UniqueObject(Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace alib */

namespace alib {

void xmlApi<void>::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Void::getXmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Void::getXmlTagName());
}

bool xmlApi<void>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, alib::Void::getXmlTagName());
}

void xmlApi<void>::compose(std::deque<sax::Token>& output) {
	output.emplace_back(Void::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	output.emplace_back(Void::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

auto voidType = registration::TypeRegister < alib::Object, alib::Void > ();

} /* namespace alib */

