/*
 * Character.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_CHARACTER_H_
#define PRIMITIVE_CHARACTER_H_

#include "PrimitiveBase.h"
#include <core/xmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Character final : public PrimitiveBase {
protected:
	char m_data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Character ( char data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	char getData ( ) const;

	explicit operator char ( ) const;

	/**
	 * sets new content of primitive
	 */
	void setData ( char data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Character & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Character";

		return xmlTagName;
	}

	static Character parse ( std::deque < sax::Token >::iterator & input );
	static char parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	static void compose ( std::deque < sax::Token > & out, char primitive );

	virtual PrimitiveBase * inc ( ) &&;

	typedef Character normalized_type;

	virtual ObjectBase * normalize ( ) && {
		return this;
	}
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < char > {
	static char parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( std::deque < sax::Token > & output, char data );
};

} /* namespace alib */

#endif /* PRIMITIVE_CHARACTER_H_ */
