/*
 * Unsigned.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Unsigned.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/TypeRegistration.hpp>

namespace primitive {

Unsigned::Unsigned(unsigned data) : m_data(data) {

}

PrimitiveBase* Unsigned::clone() const {
	return new Unsigned(*this);
}

PrimitiveBase* Unsigned::plunder() && {
	return new Unsigned(std::move(*this));
}

unsigned Unsigned::getData() const {
	return m_data;
}

Unsigned::operator unsigned ( ) const {
	return m_data;
}

Unsigned::operator size_t ( ) const {
	return m_data;
}

void Unsigned::setData(unsigned data) {
	this->m_data = data;
}

int Unsigned::compare(const Unsigned& other) const {
	return m_data - other.m_data;
}

void Unsigned::operator>>(std::ostream& out) const {
	out << "(Unsigned " << m_data << ")";
}

Unsigned::operator std::string() const {
	return std::to_string(m_data);
}

Unsigned Unsigned::parse(std::deque<sax::Token>::iterator& input) {
	return Unsigned(parseRaw(input));
}

unsigned Unsigned::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Unsigned::getXmlTagName());
	unsigned data = std::from_string < unsigned > (sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Unsigned::getXmlTagName());
	return data;
}

void Unsigned::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void Unsigned::compose(std::deque<sax::Token>& out, unsigned primitive) {
	out.emplace_back(Unsigned::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(std::to_string(primitive), sax::Token::TokenType::CHARACTER);
	out.emplace_back(Unsigned::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

PrimitiveBase * Unsigned::inc() && {
	m_data++;
	return NULL;
}

} /* namespace primitive */

namespace alib {

unsigned xmlApi<unsigned>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::Unsigned::parseRaw(input);
}

bool xmlApi<unsigned>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, xmlTagName());
}

std::string xmlApi<unsigned>::xmlTagName() {
	return primitive::Unsigned::getXmlTagName();
}

void xmlApi<unsigned>::compose(std::deque<sax::Token>& output, unsigned data) {
	primitive::Unsigned::compose(output, data);
}

auto unsignedType = registration::TypeRegister < primitive::Primitive, primitive::Unsigned > ();
auto unsignedType2 = registration::TypeRegister < alib::Object, primitive::Unsigned > ();

} /* namespace alib */

