/*
 * UnsignedLong.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "UnsignedLong.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/TypeRegistration.hpp>

namespace primitive {

UnsignedLong::UnsignedLong(unsigned long data) : m_data(data) {

}

PrimitiveBase* UnsignedLong::clone() const {
	return new UnsignedLong(*this);
}

PrimitiveBase* UnsignedLong::plunder() && {
	return new UnsignedLong(std::move(*this));
}

unsigned long UnsignedLong::getData() const {
	return m_data;
}

UnsignedLong::operator unsigned long ( ) const {
	return m_data;
}

void UnsignedLong::setData(unsigned long data) {
	this->m_data = data;
}

int UnsignedLong::compare(const UnsignedLong& other) const {
	return m_data - other.m_data;
}

void UnsignedLong::operator>>(std::ostream& out) const {
	out << "(UnsignedLong " << m_data << ")";
}

UnsignedLong::operator std::string() const {
	return std::to_string(m_data);
}

UnsignedLong UnsignedLong::parse(std::deque<sax::Token>::iterator& input) {
	return UnsignedLong(parseRaw(input));
}

unsigned long UnsignedLong::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, UnsignedLong::getXmlTagName());
	unsigned long data = std::from_string < unsigned long > (sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, UnsignedLong::getXmlTagName());
	return data;
}

void UnsignedLong::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void UnsignedLong::compose(std::deque<sax::Token>& out, unsigned long primitive) {
	out.emplace_back(UnsignedLong::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(std::to_string(primitive), sax::Token::TokenType::CHARACTER);
	out.emplace_back(UnsignedLong::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

PrimitiveBase * UnsignedLong::inc() && {
	m_data++;
	return NULL;
}

} /* namespace primitive */

namespace alib {

unsigned long xmlApi<unsigned long>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::UnsignedLong::parseRaw(input);
}

bool xmlApi<unsigned long>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ));
}

std::string xmlApi<unsigned long>::xmlTagName() {
	return primitive::UnsignedLong::getXmlTagName();
}

void xmlApi<unsigned long>::compose(std::deque<sax::Token>& output, unsigned long data) {
	primitive::UnsignedLong::compose(output, data);
}

auto unsignedLongType = registration::TypeRegister < primitive::Primitive, primitive::UnsignedLong > ();
auto unsignedLongType2 = registration::TypeRegister < alib::Object, primitive::UnsignedLong > ();

} /* namespace alib */

