/*
 * UnsignedLong.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_UNSIGNED_LONG_H_
#define PRIMITIVE_UNSIGNED_LONG_H_

#include "PrimitiveBase.h"
#include <core/xmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class UnsignedLong final : public PrimitiveBase {
protected:
	unsigned long m_data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UnsignedLong ( unsigned long data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	unsigned long getData ( ) const;

	explicit operator unsigned long ( ) const;

	/**
	 * sets new content of primitive
	 */
	void setData ( unsigned long data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UnsignedLong & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "UnsignedLong";

		return xmlTagName;
	}

	static UnsignedLong parse ( std::deque < sax::Token >::iterator & input );
	static unsigned long parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	static void compose ( std::deque < sax::Token > & out, unsigned long primitive );

	virtual PrimitiveBase * inc ( ) &&;

	typedef UnsignedLong normalized_type;

	virtual ObjectBase * normalize ( ) && {
		return this;
	}
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < unsigned long > {
	static unsigned long parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( std::deque < sax::Token > & output, unsigned long data );
};

} /* namespace alib */

#endif /* PRIMITIVE_UNSIGNED_LONG_H_ */
