#ifndef _ALGO_REGISTRATION_HPP_
#define _ALGO_REGISTRATION_HPP_

#include <introspection/Algorithms.hpp>
#include <core/multipleDispatch.hpp>

#include <abstraction/AlgorithmRegistry.hpp>
#include <abstraction/NormalizeRegistry.hpp>
#include <abstraction/DowncastRegistry.hpp>

namespace registration {

template < class Algorithm, class RealReturnType, class ... RealParameterTypeBases >
class OverloadRegister {
public:
	OverloadRegister ( typename Algorithm::template rawOverload < RealReturnType, RealParameterTypeBases ... > callback ) {
		Algorithm::template registerOverload < RealReturnType, RealParameterTypeBases ... > ( callback );

		introspection::Algorithms::registerDispatchInAlgorithm < Algorithm, RealReturnType, std::tuple < RealParameterTypeBases ... > > ( );
		introspection::Algorithms::registerAlgorithmInterface < Algorithm, typename Algorithm::returnType, typename Algorithm::frontParams, typename Algorithm::dispatchedParams, typename Algorithm::backParams > ( );
		introspection::Algorithms::registerAlgorithm < Algorithm > ( );
	}
};

template < class Type, typename enable = void >
struct RequireNormalization : std::false_type {
};

template < class Type >
struct RequireNormalization < Type, typename std::enable_if < std::has_normalize < Type >::value && ! std::is_same < Type, typename Type::normalized_type >::value >::type > : public std::true_type {
};

template < class Type, typename enable = void >
struct RequireDowncast : std::false_type {
};

template < class Type >
struct RequireDowncast < Type *, typename std::enable_if < ! std::is_final < Type >::value >::type > : public std::true_type {
};

template < class Algorithm, class ReturnType, class ... ParameterTypes >
class AbstractRegister {

	template < class Type, typename std::enable_if < RequireNormalization < Type >::value >::type * = nullptr >
	void registerNormalize ( ) {
		abstraction::NormalizeRegistry::registerNormalize < ReturnType > ( );
	}

	template < class Type, typename std::enable_if < ! RequireNormalization < Type >::value >::type * = nullptr >
	void registerNormalize ( ) {
	}

public:
	AbstractRegister ( ReturnType ( * callback ) ( ParameterTypes ... ) ) {
		bool normalize = RequireNormalization < ReturnType >::value;
		bool downcast = RequireDowncast < ReturnType >::value;

		abstraction::AlgorithmRegistry::registerAlgorithm < Algorithm, ReturnType, ParameterTypes ... > ( callback, downcast, normalize );

		registerNormalize < ReturnType > ( );
	}
};

} /* namespace registration */

#endif // _ALGO_REGISTRATION_HPP_
