#include <list>
#include "ContainerTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "object/Object.h"
#include "container/ObjectsSet.h"
#include "container/ObjectsVariant.h"
#include "container/ObjectsTree.h"
#include "container/ObjectsTrie.h"
#include "primitive/Integer.h"
#include "primitive/Character.h"
#include "primitive/String.h"

#include "factory/XmlDataFactory.hpp"

#include <type_traits>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( ContainerTest, "container" );
CPPUNIT_TEST_SUITE_REGISTRATION ( ContainerTest );

void ContainerTest::setUp ( ) {
}

void ContainerTest::tearDown ( ) {
}

void ContainerTest::testXMLParser ( ) {

	alib::Object tmp ( primitive::String ( "1" ) );

	container::ObjectsSet < > set;

	set.insert ( tmp );

	alib::Object object ( set );

	{
		std::string tmp2 = alib::XmlDataFactory::toString ( object );

		std::cout << tmp2 << std::endl;

		alib::Object object2 = alib::XmlDataFactory::fromString ( tmp2 );

		CPPUNIT_ASSERT ( object == object2 );

		std::set < primitive::String > concrete = alib::XmlDataFactory::fromString ( tmp2 );
		std::string tmp3 = alib::XmlDataFactory::toString ( concrete );

		std::cout << tmp3 << std::endl;

		alib::Object object3 = alib::XmlDataFactory::fromString ( tmp3 );

		CPPUNIT_ASSERT ( object == object3 );
	}
}

void ContainerTest::testVariantParsing ( ) {
	{
		std::string string = "<String>aaa</String>";
		std::variant < int, std::string > object = alib::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( "aaa" == object.get < std::string > ( ) );

		std::string string2 = alib::XmlDataFactory::toString ( object );

		std::variant < int, std::string > object2 = alib::XmlDataFactory::fromString ( string2 );

		CPPUNIT_ASSERT ( object == object2 );
	}
}

void ContainerTest::testTreeParsing ( ) {
	{
		std::tree < int > t ( 1 );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::tree < int > t2 = alib::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		std::tree < int > t ( 1, std::tree < int > ( 2 ), std::tree < int > ( 4 ) );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::tree < int > t2 = alib::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		std::tree < int > t ( 1, std::tree < int > ( 2, std::tree < int > ( 3 ) ), std::tree < int > ( 4 ) );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::tree < int > t2 = alib::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
}

void ContainerTest::testTrieParsing ( ) {
	{
		std::trie < char, int > t ( 1 );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::trie < char, int > t2 = alib::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		std::trie < char, int > t ( 0, std::map < char, std::trie < char, int > > { std::make_pair ( 'a', std::trie < char, int > ( 1 ) ), std::make_pair ( 'b', std::trie < char, int > ( 2 ) ) } );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::trie < char, int > t2 = alib::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		std::trie < char, int > t ( 0, std::map < char, std::trie < char, int > > { std::make_pair ( 'a', std::trie < char, int > ( 1, std::map < char, std::trie < char, int > > { std::make_pair ( 'a', std::trie < char, int > ( 3 ) ), std::make_pair ( 'b', std::trie < char, int > ( 4 ) ) } ) ), std::make_pair ( 'b', std::trie < char, int > ( 2, std::map < char, std::trie < char, int > > { std::make_pair ( 'a', std::trie < char, int > ( 5 ) ), std::make_pair ( 'b', std::trie < char, int > ( 6 ) ) } ) ) } );

		std::string string = alib::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		std::trie < char, int > t2 = alib::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
}

void ContainerTest::testProperties ( ) {
	CPPUNIT_ASSERT ( std::is_nothrow_move_constructible < container::ObjectsSet < alib::Object > >::value );
}
