/*
 * SubtreeWildcardSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef SUBTREE_WILDCARD_SYMBOL_H_
#define SUBTREE_WILDCARD_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"
#include "RankedSymbol.h"
#include <object/Object.h>

namespace alphabet {

/**
 * Represents subtreeWildcard symbol for tree linearization.
 */
class SubtreeWildcardSymbol : public SymbolBase {
public:
	/**
	 * Creates a subtreeWildcard symbol.
	 * @param symbol name of the symbol
	 */
	explicit SubtreeWildcardSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const SubtreeWildcardSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "SubtreeWildcardSymbol";

		return xmlTagName;
	}

	static SubtreeWildcardSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * inc ( ) &&;

	typedef SubtreeWildcardSymbol normalized_type;

	virtual SymbolBase * normalize ( ) && {
		return this;
	}
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type SubtreeWildcardSymbol::instance ( ) {
	return 'S';
}

template < >
inline std::string SubtreeWildcardSymbol::instance < std::string > ( ) {
	return std::string ( 1, SubtreeWildcardSymbol::instance < char > ( ) );
}

template < >
inline SubtreeWildcardSymbol SubtreeWildcardSymbol::instance < SubtreeWildcardSymbol > ( ) {
	return SubtreeWildcardSymbol ( );
}

template < >
inline Symbol SubtreeWildcardSymbol::instance < Symbol > ( ) {
	return Symbol ( SubtreeWildcardSymbol ( ) );
}

// TODO make partially specialised when needed by classes or variables, functions can be partially specialsed
template < >
inline std::ranked_symbol < > SubtreeWildcardSymbol::instance < std::ranked_symbol < > > ( ) {
	return std::ranked_symbol < > ( DefaultSymbolType ( SubtreeWildcardSymbol ( ) ), 0);
}

template < >
inline RankedSymbol < > SubtreeWildcardSymbol::instance < RankedSymbol < > > ( ) {
	return RankedSymbol < > ( SubtreeWildcardSymbol::instance < std::ranked_symbol < > > ( ) );
}

template < >
inline alib::Object SubtreeWildcardSymbol::instance < alib::Object > ( ) {
	return alib::Object ( SubtreeWildcardSymbol ( ) );
}

} /* namespace alphabet */

#endif /* SUBTREE_WILDCARD_SYMBOL_H_ */
