/*
 * RTEFromXMLParser.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef RTE_FROM_XML_PARSER_H_
#define RTE_FROM_XML_PARSER_H_

#include <set>
#include <deque>
#include "../RTEFeatures.h"
#include <sax/Token.h>

#include <alphabet/ranked_symbol.hpp>

#include "../formal/FormalRTEElements.h"
#include <core/xmlApi.hpp>

namespace rte {

/**
 * Parser used to get RTE from XML parsed into list of tokens.
 */
class RTEFromXMLParser {
public:
	template < class SymbolType, class RankType >
	static std::pair < std::set < std::ranked_symbol < SymbolType, RankType > >, std::set < std::ranked_symbol < SymbolType, RankType > > > parseAlphabet ( std::deque < sax::Token >::iterator & input );

	template < class SymbolType, class RankType >
	static std::rvalue_ref < FormalRTEElement < SymbolType, RankType > > parseFormalRTEElement ( std::deque < sax::Token >::iterator & input );

	template < class SymbolType, class RankType >
	static std::rvalue_ref < FormalRTEEmpty < SymbolType, RankType > > parseFormalRTEEmpty ( std::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static std::rvalue_ref < FormalRTEIteration < SymbolType, RankType > > parseFormalRTEIteration ( std::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static std::rvalue_ref < FormalRTEAlternation < SymbolType, RankType > > parseFormalRTEAlternation ( std::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static std::rvalue_ref < FormalRTESubstitution < SymbolType, RankType > > parseFormalRTESubstitution ( std::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static std::rvalue_ref < FormalRTESymbolAlphabet < SymbolType, RankType > > parseFormalRTESymbolAlphabet ( std::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static std::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > parseFormalRTESymbolSubst ( std::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static std::rvalue_ref < FormalRTESymbol < SymbolType, RankType > > parseFormalRTESymbol ( std::deque < sax::Token >::iterator & input );
};

template < class SymbolType, class RankType >
std::pair < std::set < std::ranked_symbol < SymbolType, RankType > >, std::set < std::ranked_symbol < SymbolType, RankType > > > RTEFromXMLParser::parseAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < std::ranked_symbol < SymbolType, RankType > > alphabetF, alphabetK;

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabetF.insert ( alib::xmlApi < std::ranked_symbol < SymbolType, RankType > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbolAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabetK.insert ( alib::xmlApi < std::ranked_symbol < SymbolType, RankType > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substSymbolAlphabet" );

	return std::make_pair ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
std::rvalue_ref < FormalRTEElement < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTEElement ( std::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" ) )
		return parseFormalRTEEmpty < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" ) )
		return parseFormalRTEIteration < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" ) )
		return parseFormalRTEAlternation < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "substitution" ) )
		return parseFormalRTESubstitution < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" ) )
		return parseFormalRTESymbolAlphabet < SymbolType, RankType > ( input );
	else
		return parseFormalRTESymbolSubst < SymbolType, RankType > ( input );
}

template < class SymbolType, class RankType >
std::rvalue_ref < FormalRTEAlternation < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTEAlternation ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" );

	std::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element1 = parseFormalRTEElement < SymbolType, RankType > ( input );
	std::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element2 = parseFormalRTEElement < SymbolType, RankType > ( input );

	std::rvalue_ref < FormalRTEAlternation < SymbolType, RankType > > alternation ( new FormalRTEAlternation < SymbolType, RankType > ( std::move ( element1 ), std::move ( element2 ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alternation" );
	return alternation;
}

template < class SymbolType, class RankType >
std::rvalue_ref < FormalRTESubstitution < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTESubstitution ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substitution" );

	std::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > ssymb ( new FormalRTESymbolSubst < SymbolType, RankType > ( alib::xmlApi < std::ranked_symbol < SymbolType, RankType > >::parse ( input ) ) );
	std::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element1 = parseFormalRTEElement < SymbolType, RankType > ( input );
	std::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element2 = parseFormalRTEElement < SymbolType, RankType > ( input );

	std::rvalue_ref < FormalRTESubstitution < SymbolType, RankType > > substitution ( new FormalRTESubstitution < SymbolType, RankType > ( std::move ( element1 ), std::move ( element2 ), std::move ( ssymb ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substitution" );
	return substitution;
}

template < class SymbolType, class RankType >
std::rvalue_ref < FormalRTEIteration < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTEIteration ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" );

	std::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > ssymb ( new FormalRTESymbolSubst < SymbolType, RankType > ( alib::xmlApi < std::ranked_symbol < SymbolType, RankType > >::parse ( input ) ) );
	std::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element = parseFormalRTEElement < SymbolType, RankType > ( input );
	std::rvalue_ref < FormalRTEIteration < SymbolType, RankType > > iteration ( new FormalRTEIteration < SymbolType, RankType > ( std::move ( element ), std::move ( ssymb ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "iteration" );
	return iteration;
}

template < class SymbolType, class RankType >
std::rvalue_ref < FormalRTEEmpty < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTEEmpty ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" );

	std::rvalue_ref < FormalRTEEmpty < SymbolType, RankType > > empty ( new FormalRTEEmpty < SymbolType, RankType > ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "empty" );

	return empty;
}

template < class SymbolType, class RankType >
std::rvalue_ref < FormalRTESymbol < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTESymbol ( std::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" ) )
		return parseFormalRTESymbolAlphabet < SymbolType, RankType > ( input );
	else
		return parseFormalRTESymbolSubst < SymbolType, RankType > ( input );
}

template < class SymbolType, class RankType >
std::rvalue_ref < FormalRTESymbolAlphabet < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTESymbolAlphabet ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" );
	std::ranked_symbol < SymbolType, RankType > symbol = alib::xmlApi < std::ranked_symbol < SymbolType, RankType > >::parse ( input );
	std::vector < std::rvalue_ref < FormalRTESymbol < SymbolType, RankType > > > elements;

	std::rvalue_ref < FormalRTESymbolAlphabet < SymbolType, RankType > > ret ( new FormalRTESymbolAlphabet < SymbolType, RankType > ( symbol ) );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		elements.push_back ( parseFormalRTESymbol < SymbolType, RankType > ( input ) );

	for ( std::rvalue_ref < FormalRTESymbol < SymbolType, RankType > > & element : elements )
		ret->appendElement ( element );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "symbol" );

	return ret;
}

template < class SymbolType, class RankType >
std::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTESymbolSubst ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbol" );
	std::ranked_symbol < SymbolType, RankType > symbol = alib::xmlApi < std::ranked_symbol < SymbolType, RankType > >::parse ( input );

	std::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > ret ( new FormalRTESymbolSubst < SymbolType, RankType > ( symbol ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substSymbol" );

	return ret;
}

} /* namespace rte */

#endif /* RTE_FROM_XML_PARSER_H_ */
