/*
 * StringToXMLComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnciek
 */

#ifndef STRING_TO_XML_COMPOSER_H_
#define STRING_TO_XML_COMPOSER_H_

#include <deque>
#include <sax/Token.h>
#include <set>
#include <vector>
#include <core/xmlApi.hpp>

namespace string {

/**
 * This class contains methods to print XML representation of string to the output stream.
 */
class StringToXMLComposer {
public:
	template < class SymbolType >
	static void compose(std::deque<sax::Token>& out, const std::set<SymbolType>& alphabet);
	template < class SymbolType >
	static void compose(std::deque<sax::Token>& out, const std::vector<SymbolType>& content);
};

template < class SymbolType >
void StringToXMLComposer::compose(std::deque<sax::Token>& out, const std::set<SymbolType>& alphabet) {
	out.emplace_back("alphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : alphabet) {
		alib::xmlApi<SymbolType>::compose(out, symbol);
	}
	out.emplace_back("alphabet", sax::Token::TokenType::END_ELEMENT);
}

template < class SymbolType >
void StringToXMLComposer::compose(std::deque<sax::Token>& out, const std::vector<SymbolType>& content) {
	out.emplace_back("content", sax::Token::TokenType::START_ELEMENT);
	for(const auto& symbol : content) {
		alib::xmlApi<SymbolType>::compose(out, symbol);
	}
	out.emplace_back("content", sax::Token::TokenType::END_ELEMENT);
}

} /* namespace string */

#endif /* STRING_TO_XML_COMPOSER_H_ */
