/*
 * TreeNormalize.h
 *
 *  Created on: Mar 31, 2017
 *      Author: Jan Travnicek
 */

#ifndef TREE_NORMALIZE_H_
#define TREE_NORMALIZE_H_

#include <tree>

#include <alphabet/RankedSymbol.h>
#include <alphabet/common/SymbolNormalize.h>

namespace tree {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class TreeNormalize {
public:
	template < class SymbolType >
	static std::tree < DefaultSymbolType > normalizeTree ( std::tree < SymbolType > && tree );

	template < class SymbolType, class RankType >
	static std::tree < std::ranked_symbol < DefaultSymbolType, DefaultRankType > > normalizeRankedTree ( std::tree < std::ranked_symbol < SymbolType, RankType > > && tree );

};

template < class SymbolType >
std::tree < DefaultSymbolType > TreeNormalize::normalizeTree ( std::tree < SymbolType > && tree ) {
	std::vector < std::tree < DefaultSymbolType > > children;

	for ( std::tree < SymbolType > & child : tree.getChildren ( ) ) {
		children.push_back ( normalizeTree ( std::move ( child ) ) );
	}

	return std::tree < DefaultSymbolType > ( alphabet::SymbolNormalize::normalizeSymbol ( std::move ( tree.getData ( ) ) ), std::move ( children ) );
}

template < class SymbolType, class RankType >
std::tree < std::ranked_symbol < DefaultSymbolType, DefaultRankType > > TreeNormalize::normalizeRankedTree ( std::tree < std::ranked_symbol < SymbolType, RankType > > && tree ) {
	std::vector < std::tree < std::ranked_symbol < DefaultSymbolType, DefaultRankType > > > children;

	for ( std::tree < std::ranked_symbol < SymbolType, RankType > > & child : tree.getChildren ( ) ) {
		children.push_back ( normalizeRankedTree ( std::move ( child ) ) );
	}

	return std::tree < std::ranked_symbol < DefaultSymbolType, DefaultRankType > > ( alphabet::SymbolNormalize::normalizeRankedSymbol ( std::move ( tree.getData ( ) ) ), std::move ( children ) );
}

} /* namespace tree */

#endif /* TREE_NORMALIZE_H_ */

