/*
 * GraphToXMLComposer.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#include "GraphToXMLComposer.h"
#include <core/xmlApi.hpp>

#include "../GraphTypes.h"
#include "../Graph.h"
#include "../GraphClasses.h"

namespace graph {

void GraphToXMLComposer::composeNode(std::deque<sax::Token>& out, const Node &node) {
	out.emplace_back("node", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<label::Label>::compose(out, node.getName());
	out.emplace_back("node", sax::Token::TokenType::END_ELEMENT);
}

void GraphToXMLComposer::composeEdge(std::deque<sax::Token>& out, const DirectedEdge &edge) {
	out.emplace_back("edge", sax::Token::TokenType::START_ELEMENT);

	out.emplace_back("from", sax::Token::TokenType::START_ELEMENT);
	composeNode(out, edge.getFromNode());
	out.emplace_back("from", sax::Token::TokenType::END_ELEMENT);

	alib::xmlApi<label::Label>::compose(out, edge.getName());

	out.emplace_back("to", sax::Token::TokenType::START_ELEMENT);
	composeNode(out, edge.getToNode());
	out.emplace_back("to", sax::Token::TokenType::END_ELEMENT);

	out.emplace_back("edge", sax::Token::TokenType::END_ELEMENT);
}

void GraphToXMLComposer::composeEdge(std::deque<sax::Token>& out, const UndirectedEdge &edge) {
	out.emplace_back("edge", sax::Token::TokenType::START_ELEMENT);

	out.emplace_back("first", sax::Token::TokenType::START_ELEMENT);
	composeNode(out, edge.getFirstNode());
	out.emplace_back("first", sax::Token::TokenType::END_ELEMENT);

	alib::xmlApi<label::Label>::compose(out, edge.getName());

	out.emplace_back("second", sax::Token::TokenType::START_ELEMENT);
	composeNode(out, edge.getSecondNode());
	out.emplace_back("second", sax::Token::TokenType::END_ELEMENT);

	out.emplace_back("edge", sax::Token::TokenType::END_ELEMENT);
}

void GraphToXMLComposer::composeRepresentation(std::deque<sax::Token> &out, const std::string &representation) {
	out.emplace_back("representation", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(representation, sax::Token::TokenType::CHARACTER);
	out.emplace_back("representation", sax::Token::TokenType::END_ELEMENT);
}

void GraphToXMLComposer::composeNodes(std::deque<sax::Token> &out, const std::set<Node> &nodes) {
	out.emplace_back("nodes", sax::Token::TokenType::START_ELEMENT);
	for (const Node &node : nodes) {
		composeNode(out, node);
	}
	out.emplace_back("nodes", sax::Token::TokenType::END_ELEMENT);
}

} // namespace graph
