/*
 * DirectedEdge.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef DIRECTED_EDGE_H_
#define DIRECTED_EDGE_H_

#include "../common/Macros.h"
#include "../common/Node.h"

namespace graph {

class DirectedEdge {
	Node m_from;
	Node m_to;
	label::Label m_name;

	void dump(std::ostream & os) const;

public:
	/// should be used only to create variable, to which will be assigned soon
	DirectedEdge() : m_from(), m_to(), m_name(label::Label("uninitialized")) {}

	explicit DirectedEdge(Node from, Node to);
	explicit DirectedEdge(Node from, Node to, label::Label name);
	explicit DirectedEdge(Node from, Node to, int number);
	explicit DirectedEdge(Node from, Node to, char character);
	explicit DirectedEdge(Node from, Node to, std::string name);

	DirectedEdge * clone() const;
	DirectedEdge * plunder() &&;

	const Node & getFromNode() const;
	const Node & getToNode() const;
	const label::Label & getName() const;

	bool operator < (const DirectedEdge & other) const;
	bool operator > (const DirectedEdge & other) const;
	bool operator == (const DirectedEdge & other) const;
	bool operator != (const DirectedEdge & other) const;
	int compare(const DirectedEdge & other) const;

	friend std::ostream &operator<<(std::ostream & out, const DirectedEdge & node);
	explicit operator std::string() const;
};

} // namespace graph

namespace std {

template<>
struct compare<graph::DirectedEdge> {
	int operator()(const graph::DirectedEdge& first, const graph::DirectedEdge& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

GRAPH_DEFINE_STD_HASH(DirectedEdge)

#endif // DIRECTED_EDGE_H_
