/*
 * Author: Radovan Cerveny
 */

#ifndef MEASUREMENT_PROCESSOR_OUTPUT_HPP_
#define MEASUREMENT_PROCESSOR_OUTPUT_HPP_

#include "../provisioner/MeasurementProvisionerResults.hpp"
#include <ostream>
#include <set>

namespace measurements {

struct MeasurementProcessorOutput {
	enum class OutputFormat {
		XML, STATS,	CSV, HTML
	};

	enum class OutputEngine {
		TIME, MEMORY, COUNTER
	};

	OutputFormat			 outputFormat;
	OutputEngine			 outputEngine;
	std::set < std::string > outputEngineAttr;

	static void outputXml ( std::ostream &, const MeasurementProvisionerResults &, const MeasurementProcessorOutput & );
	static void outputStats ( std::ostream &, const MeasurementProvisionerResults &, const MeasurementProcessorOutput & );
	static void outputCsv ( std::ostream &, const MeasurementProvisionerResults &, const MeasurementProcessorOutput & );
	static void outputHtml ( std::ostream &, const MeasurementProvisionerResults &, const MeasurementProcessorOutput & );

private:
	struct Row {
		std::string					input;
		std::vector < std::string > data;
	};

	struct Table {
		std::string					description;
		std::vector < std::string > header;
		std::vector < Row >			rows;
	};

	static std::vector < Table > getTableRepresentation ( const MeasurementProvisionerResults &, const MeasurementProcessorOutput & );
	static std::string			 getData ( const MeasurementFrame &, const MeasurementProcessorOutput & );
};

}

#endif /* MEASUREMENT_PROCESSOR_OUTPUT_HPP_ */
