/*
 * Author: Radovan Cerveny
 */

#include "MeasurementProvisionerUtils.hpp"
#include "exception/CommonException.h"
#include <wordexp.h>
#include <unistd.h>
#include <iostream>
#include <random>
#include <string>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>

namespace measurements {

std::list < std::string > MPUtils::shellExpand ( const std::string & str ) {
	std::list < std::string > results;

	wordexp_t wexp;

	if ( wordexp ( str.c_str ( ), & wexp, 0 ) != 0 )
		throw::exception::CommonException ( "MPUtils: wordexp failed" );

	for ( unsigned i = 0; i < wexp.we_wordc; ++i )
		results.emplace_back ( wexp.we_wordv[i] );

	wordfree ( & wexp );

	return results;
}

std::string MPUtils::randomFilename ( ) {
	static std::string randomSeed = std::to_string ( std::random_devices::semirandom ( ) );
	static int randomOrder = 0;

	return randomSeed + "_" + std::to_string ( randomOrder++ );
}

MPUtils::ShmFileHandle MPUtils::openShmFile ( ) {

	std::string filename = "/" + randomFilename ( );

	int shmFd = shm_open ( filename.c_str ( ), O_RDWR | O_CREAT, S_IRWXU );

	if ( shmFd == -1 )
		throw::exception::CommonException ( "MPUtils: shm_open failed" );

	 // turn off FD_CLOEXEC, so it survives call to system() (exec())
	int fl = fcntl ( shmFd, F_GETFD );
	fl &= ~FD_CLOEXEC;
	fcntl ( shmFd, F_SETFD, fl );

	filename = "/dev/shm" + filename;

	if ( access ( filename.c_str ( ), F_OK ) == -1 ) {
		shm_unlink ( filename.c_str ( ) );
		throw::exception::CommonException ( "MPUtils: can't create shm file in: " + filename );
	}

	return {
			   shmFd, std::move ( filename )
	};
}

void MPUtils::closeShmFile ( const ShmFileHandle & shmFileHandle ) {
	unlink ( shmFileHandle.filename.c_str ( ) );
}

std::string MPUtils::generateTmpfileFromCommand ( const std::string & command ) {
	ShmFileHandle tempShmFile = openShmFile ( );

	int stdoutfd = dup ( 1 );

	if ( stdoutfd == -1 )
		throw::exception::CommonException ( "MeasurementProvisionerUtils: dup failed" );

	close ( 1 );

	if ( dup ( tempShmFile.fd ) == -1 )
		throw::exception::CommonException ( "MeasurementProvisionerUtils: dup failed" );

	int status = system ( command.c_str ( ) );

	close ( 1 );
	close ( tempShmFile.fd );

	if ( dup ( stdoutfd ) == -1 )
		throw::exception::CommonException ( "MeasurementProvisionerUtils: dup failed" );

	close ( stdoutfd );

	if ( WEXITSTATUS ( status ) != 0 ) {
		closeShmFile ( tempShmFile );
		throw::exception::CommonException ( "MPUtils: system call failed" );
	}

	return tempShmFile.filename;
}

}
