/*
 * string.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __STRING_HPP_
#define __STRING_HPP_

#include <string>
#include <stdexcept>
#include "compare.hpp"

namespace std {

template<typename... Ts>
using to_string_type = auto(Ts...) -> decltype(to_string__private(std::declval<Ts>()...));

template < typename T, typename std::enable_if < std::is_function< to_string_type < T > >::value >::type* = nullptr >
string to_string ( T value ) {
	return to_string__private ( value );
}

template < typename T, typename std::enable_if < std::is_constructible < string, T >::value >::type* = nullptr >
string to_string ( const T & value ) {
	return (string) value;
}

template < >
string to_string ( const string & value );

template < typename T >
T from_string ( const string & );

template < >
string from_string ( const string & value );

template < >
int from_string ( const string & value );

template < >
long from_string ( const string & value );

template < >
long long from_string ( const string & value );

template < >
unsigned from_string ( const string & value );

template < >
unsigned long from_string ( const string & value );

template < >
unsigned long long from_string ( const string & value );

template < >
double from_string ( const string & value );

template < >
struct compare < string > {
	int operator ()( const string & first, const string & second ) const {
		return first.compare ( second );
	}

};

string cstringToString ( char * param );

} /* namespace std */

#endif /* __STRING_HPP_ */
