/*
 * type_traits.hpp
 *
 * Created on: Feb 28, 2014
 * Author: Jan Travnicek
 */

#ifndef __TYPE_TRAITS_HPP_
#define __TYPE_TRAITS_HPP_

#include <type_traits>
#include <utility>

namespace std {

	template<class T>
	struct has_clone {
		typedef char (&Yes)[1];
		typedef char (&No)[2];

		template<class U>
		static Yes test(U * data, typename std::enable_if< std::is_pointer<decltype(data->clone())>::value>::type * = 0);
		static No test(...);
		static const bool value = sizeof(Yes) == sizeof(has_clone::test((typename std::remove_reference<T>::type*)0));
	};

	template<class T>
	struct has_normalize {
		typedef char (&Yes)[1];
		typedef char (&No)[2];

		template<class U>
		static Yes test(U * data, typename std::enable_if< std::is_pointer<decltype(std::move ( * data ).normalize())>::value>::type * = 0);
		static No test(...);
		static const bool value = sizeof(Yes) == sizeof(has_normalize::test((typename std::remove_reference<T>::type*)0));
	};

// ----------------------------------------------------------------------------------------------------

	template <typename T>
	struct is_base_of<T, T> {
		static const bool value = true;
	};

	template <typename T, typename... Ts>
	struct is_base_of_any;

	template <typename T, typename F>
	struct is_base_of_any<T, F>
	{
		static const bool value = is_base_of<T, F>::value;
	};

	template <typename T, typename F, typename... Ts>
	struct is_base_of_any<T, F, Ts...>
	{
		static const bool value = is_base_of<T, F>::value || is_base_of_any<T, Ts...>::value;
	};

// ----------------------------------------------------------------------------------------------------

	template < size_t N, typename ... T >
	struct get_type_pack_element;

	template < typename T, typename ... Ts>
	struct get_type_pack_element < 0, T, Ts ... > {
		typedef T type;
	};

	template < size_t N, typename T, typename ... Ts >
	struct get_type_pack_element < N, T, Ts ... > {
		typedef typename get_type_pack_element < N - 1, Ts ... >::type type;
	};

// ----------------------------------------------------------------------------------------------------

	template < typename T, typename ... Ts >
	struct is_in;

	template < typename T >
	struct is_in < T > : std::false_type { };

	template < typename T, typename U, typename ... Ts >
	struct is_in < T, U, Ts ... > : is_in < T, Ts ... > { };

	template < typename T, typename ... Ts >
	struct is_in < T, T, Ts ... > : std::true_type { };

// ----------------------------------------------------------------------------------------------------

	template < class Ref, class Type >
	struct match_cv_ref_rec {
		typedef Type type;
	};

	template < class Ref, class Type >
	struct match_cv_ref_rec < const Ref, Type > {
		typedef typename add_const < typename match_cv_ref_rec < Ref, Type >::type >::type type;
	};

	template < class Ref, class Type >
	struct match_cv_ref_rec < Ref &, Type > {
		typedef typename add_lvalue_reference < typename match_cv_ref_rec < Ref, Type >::type >::type type;
	};

	template < class Ref, class Type >
	struct match_cv_ref_rec < Ref &&, Type > {
		typedef typename add_rvalue_reference < typename match_cv_ref_rec < Ref, Type >::type >::type type;
	};

	template < class Ref, class Type >
	struct match_cv_ref_rec < volatile Ref, Type > {
		typedef typename add_volatile < typename match_cv_ref_rec < Ref, Type >::type >::type type;
	};

	template < class Ref, class Type >
	struct match_cv_ref {
		typedef typename match_cv_ref_rec < Ref, typename decay < Type >::type >::type type;
	};

// ----------------------------------------------------------------------------------------------------

	template < typename ... T >
	struct all_same : std::false_type { };

	template < >
	struct all_same < > : std::true_type { };

	template < typename T >
	struct all_same < T > : std::true_type { };

	template < typename T, typename ... Ts >
	struct all_same < T, T, Ts ... > : all_same < T, Ts ... > { };

} /* namespace std */

#endif /* __TYPE_TRAITS_HPP_ */
