/*
 * utility.hpp
 *
 * Created on: Apr 7, 2016
 * Author: Jan Travnicek
 */

#ifndef __UTILITY_HPP_
#define __UTILITY_HPP_

#include <utility>
#include "type_traits.hpp"

namespace std {

template < class T, typename std::enable_if < ! std::has_clone < T >::value >::type * = nullptr >
inline T move_copy ( const T & param ) {
	return T ( param );
}

template < class T >
struct rvalue_ref {
	T * holder;

	explicit rvalue_ref ( T * ptr ) : holder ( ptr ) {

	}

	template < class U >
	rvalue_ref ( rvalue_ref < U > && other ) : holder ( other.holder ) {
		other.holder = nullptr;
	}

	rvalue_ref ( const rvalue_ref & ) = delete;

	rvalue_ref ( rvalue_ref && other) : holder( other.holder ) {
		other.holder = nullptr;
	};

	rvalue_ref & operator = ( const rvalue_ref & ) = delete;

	rvalue_ref & operator = ( rvalue_ref && other ) {
		std::swap(holder, other.holder);
	}

	~rvalue_ref ( ) {
		delete holder;
	}

	T * operator ->( ) {
		return holder;
	}

	T * operator ->( ) const {
		return holder;
	}

	operator T && ( ) {
		return std::move ( * holder );
	}
};

template < class T >
inline rvalue_ref < T > manage_move ( T * param ) {
	return rvalue_ref < T > { param };
}

template < class T, typename std::enable_if < std::has_clone < T >::value >::type * = nullptr >
inline rvalue_ref < T > move_copy ( const T & param ) {
	return manage_move ( param.clone ( ) );
}

 // TODO remove after switching to c++17 (as_const)
template < class T >
inline const T & make_const ( T & ptr ) {
	return static_cast < const T & > ( ptr );
}

template < class T >
inline const T & make_const ( const T & ptr ) {
	return ptr;
}

} /* namespace std */

#endif /* __UTILITY_HPP_ */
