#ifndef TYPE_TRAITS_TEST_H_
#define TYPE_TRAITS_TEST_H_

#include <cppunit/extensions/HelperMacros.h>

class SharedPtrTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( SharedPtrTest );
  CPPUNIT_TEST( testSharedPtr1 );
  CPPUNIT_TEST( testSharedPtr2 );
  CPPUNIT_TEST( testSharedPtr3 );
  CPPUNIT_TEST_SUITE_END();

public:
class Moveable {
	int& m_moves;
	int& m_copies;

public:
	Moveable(int& moves, int& copies) : m_moves(moves), m_copies(copies) {
		m_moves = 0;
		m_copies = 0;
	}

	Moveable(const Moveable& src) : m_moves(src.m_moves), m_copies(src.m_copies) {
		m_copies++;
	}

	Moveable(Moveable&& src) : m_moves(src.m_moves), m_copies(src.m_copies) {
		m_moves++;
	}

	bool operator<(const Moveable&) const {
		return false;
	}
};

class Moveable2 : public std::cow_shared_ptr_base {
	int& m_moves;
	int& m_copies;

public:
	Moveable2(int& moves, int& copies) : m_moves(moves), m_copies(copies) {
		m_moves = 0;
		m_copies = 0;
	}

	Moveable2(const Moveable2& src) : m_moves(src.m_moves), m_copies(src.m_copies) {
		m_copies++;
	}

	Moveable2(Moveable2&& src) : m_moves(src.m_moves), m_copies(src.m_copies) {
		m_moves++;
	}

	bool operator<(const Moveable2&) const {
		return false;
	}
};

class Base {
public:
	virtual Base* clone() const = 0;

};

class Derived : public Base {
public:
	virtual Derived* clone() const override {
		return new Derived(*this);
	}

};

public:
  void setUp();
  void tearDown();

  void testSharedPtr1();
  void testSharedPtr2();
  void testSharedPtr3();
};

#endif  // TYPE_TRAITS_TEST_H_
