#ifndef TREE_TEST_BASE_H_
#define TREE_TEST_BASE_H_

#include <cppunit/extensions/HelperMacros.h>
#include <compare>
#include <tree>

class TreeBaseTest : public CppUnit::TestFixture {
	CPPUNIT_TEST_SUITE ( TreeBaseTest );
	CPPUNIT_TEST ( testRegexps );
	CPPUNIT_TEST ( testRankedTree );
	CPPUNIT_TEST ( testUnrankedTree );
	CPPUNIT_TEST_SUITE_END ( );

public:
	class RegExpElement : public std::BaseNode < RegExpElement > {
	};

	class RegExpAlternation : public RegExpElement {
	};

	class RegExpConcatenation : public RegExpElement, public std::BinaryNode < std::shared_ptr < RegExpElement >, std::shared_ptr < const RegExpElement >, RegExpConcatenation > {
	public:
		RegExpConcatenation ( std::shared_ptr < RegExpElement > left, std::shared_ptr < RegExpElement > right ) : BinaryNode < std::shared_ptr < RegExpElement >, std::shared_ptr < const RegExpElement >, RegExpConcatenation > ( std::move ( left ), std::move ( right ) ) {
		}

	};

	class RegExpIteration : public RegExpElement, public std::UnaryNode < std::shared_ptr < RegExpElement >, std::shared_ptr < const RegExpElement >, RegExpIteration > {
	public:
		RegExpIteration ( std::shared_ptr < RegExpElement > element ) : UnaryNode < std::shared_ptr < RegExpElement >, std::shared_ptr < const RegExpElement >, RegExpIteration > ( std::move ( element ) ) {
		}

	};

	class RegExpSymbol : public RegExpElement, public std::NullaryNode < std::shared_ptr < RegExpElement >, std::shared_ptr < const RegExpElement >, RegExpSymbol > {
		char m_symbol;

	public:
		RegExpSymbol ( char symbol ) : m_symbol ( symbol ) {
		}

		char getSymbol ( ) const {
			return m_symbol;
		}

	};

	class RegExpEpsilon : public RegExpElement {
	};

	class RegExpEmpty : public RegExpElement {
	};

	class RegExp {
		std::shared_ptr < RegExpElement > root;
	};

	class UnrankedTreeNode : public std::BaseNode < UnrankedTreeNode >, public std::VararyNode < UnrankedTreeNode > {
		char m_symbol;

	public:
		UnrankedTreeNode ( char symbol, std::vector < UnrankedTreeNode > c ) : VararyNode < UnrankedTreeNode > ( std::move ( c ) ), m_symbol ( symbol ) {
		}

		char getSymbol ( ) const {
			return m_symbol;
		}

	};

	class UnrankedTree {
		UnrankedTreeNode root;
	};

	class RankedTreeNode : public std::BaseNode < RankedTreeNode >, public std::FixedaryNode < RankedTreeNode > {
		char m_symbol;
		unsigned m_arity;

	public:
		RankedTreeNode ( char symbol, unsigned arity, std::vector < RankedTreeNode > c ) : FixedaryNode < RankedTreeNode > ( std::move ( c ) ), m_symbol ( symbol ), m_arity ( arity ) {
			if ( getChildren ( ).size ( ) != m_arity )
				throw "Arity != size";
		}

		char getSymbol ( ) const {
			return m_symbol;
		}

	};

	class RankedTree {
		RankedTreeNode root;
	};

public:
	void setUp ( );
	void tearDown ( );

	void testRegexps ( );
	void testRankedTree ( );
	void testUnrankedTree ( );
};

#endif // TREE_TEST_BASE_H_
