/*
 * AutomatonFromStringParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_FROM_STRING_PARSER_H_
#define AUTOMATON_FROM_STRING_PARSER_H_

#include "AutomatonFromStringLexer.h"

#include "automaton/AutomatonFeatures.h"
#include "string/StringFeatures.h"
#include "alphabet/SymbolFeatures.h"
#include "label/LabelFeatures.h"

#include <set>
#include <vector>
#include <tuple>
#include <variant>

namespace automaton {

class AutomatonFromStringParser {
	static void initialFinalState(std::istream& input, bool& initial, bool& final);

	static void parseEpsilonNFATransition(std::istream& input, std::set<DefaultStateType>& states, const std::vector<std::variant<DefaultEpsilonType, DefaultSymbolType>>& symbols, DefaultStateType*& initialState, std::set<DefaultStateType>& finalStates, std::set<std::tuple<DefaultStateType, std::variant<DefaultEpsilonType, DefaultSymbolType>, DefaultStateType>>& transitionFunction);
	static void parseMultiInitialStateNFATransition(std::istream& input, MultiInitialStateNFA < DefaultSymbolType, DefaultStateType > & res, const std::vector<DefaultSymbolType>& symbols);
	static void parseNFATransition(std::istream& input, std::set<DefaultStateType>& states, const std::vector<DefaultSymbolType>& symbols, DefaultStateType*& initialState, std::set<DefaultStateType>& finalStates, std::set<std::tuple<DefaultStateType, DefaultSymbolType, DefaultStateType>>& transitionFunction);
	static void parseDFATransition(std::istream& input, std::set<DefaultStateType>& states, const std::vector<DefaultSymbolType>& symbols, DefaultStateType*& initialState, std::set<DefaultStateType>& finalStates, std::set<std::tuple<DefaultStateType, DefaultSymbolType, DefaultStateType>>& transitionFunction);

public:
	static AutomatonBase * parseAutomaton ( std::string string );
	static Automaton parseAutomaton(std::istream& input);
	static Automaton parseAutomaton(std::istream& input, const std::set<FEATURES>& features);

	static EpsilonNFA< > parseEpsilonNFA(std::istream& input);
	static NFA< > parseNFA(std::istream& input);
	static MultiInitialStateNFA < > parseMultiInitialStateNFA(std::istream& input);
	static DFA < > parseDFA(std::istream& input);
};

} /* namespace automaton */

#endif /* AUTOMATON_FROM_STRING_PARSER_H_ */
