/*
 * GrammarToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarToStringComposer.h"
#include "grammar/GrammarClasses.h"
#include <registration/AlgoRegistration.hpp>

namespace grammar {

void GrammarToStringComposer::compose(std::ostream& output, const Grammar& grammar) {
	dispatch(output, grammar.getData());
}

auto ObjectToStringComposerGrammar = registration::OverloadRegister < alib::WrapperBaseDispatcher, void, Grammar>(GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const LeftLG < SymbolType > & grammar) {
	output << "LEFT_LG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerLeftLG = registration::OverloadRegister < GrammarToStringComposer, void, LeftLG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerLeftLG = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, LeftLG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const LeftRG < SymbolType > & grammar) {
	output << "LEFT_RG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerLeftRG = registration::OverloadRegister < GrammarToStringComposer, void, LeftRG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerLeftRG = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, LeftRG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const RightLG < SymbolType > & grammar) {
	output << "RIGHT_LG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerRightLG = registration::OverloadRegister < GrammarToStringComposer, void, RightLG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerRightLG = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, RightLG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const RightRG < SymbolType > & grammar) {
	output << "RIGHT_RG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerRightRG = registration::OverloadRegister < GrammarToStringComposer, void, RightRG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerRightRG = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, RightRG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const LG < SymbolType > & grammar) {
	output << "LG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerLG = registration::OverloadRegister < GrammarToStringComposer, void, LG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerLG = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, LG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const CFG < SymbolType > & grammar) {
	output << "CFG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerCFG = registration::OverloadRegister < GrammarToStringComposer, void, CFG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerCFG = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, CFG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const EpsilonFreeCFG < SymbolType > & grammar) {
	output << "EPSILON_FREE_CFG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerEpsilonFreeCFG = registration::OverloadRegister < GrammarToStringComposer, void, EpsilonFreeCFG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerEpsilonFreeCFG = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, EpsilonFreeCFG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const CNF < SymbolType > & grammar) {
	output << "CNF";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerCNF = registration::OverloadRegister < GrammarToStringComposer, void, CNF < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerCNF = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, CNF < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const GNF < SymbolType > & grammar) {
	output << "GNF";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerGNF = registration::OverloadRegister < GrammarToStringComposer, void, GNF < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerGNF = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, GNF < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const CSG < SymbolType > & grammar) {
	output << "CSG";
	composePreservingCSLikeGrammar(output, grammar);
}

auto GrammarToStringComposerCSG = registration::OverloadRegister < GrammarToStringComposer, void, CSG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerCSG = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, CSG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const NonContractingGrammar < SymbolType > & grammar) {
	output << "NON_CONTRACTING_GRAMMAR";
	composeCSLikeGrammar(output, grammar);
}

auto GrammarToStringComposerNonContractingGrammar = registration::OverloadRegister < GrammarToStringComposer, void, NonContractingGrammar < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerNonContractingGrammar = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, NonContractingGrammar < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const ContextPreservingUnrestrictedGrammar < SymbolType > & grammar) {
	output << "CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR";
	composePreservingCSLikeGrammar(output, grammar);
}

auto GrammarToStringComposerContextPreservingUnrestructedGrammar = registration::OverloadRegister < GrammarToStringComposer, void, ContextPreservingUnrestrictedGrammar < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerContextPreservingUnrestrictedGrammar = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, ContextPreservingUnrestrictedGrammar < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const UnrestrictedGrammar < SymbolType > & grammar) {
	output << "UNRESTRICTED_GRAMMAR";
	composeCSLikeGrammar(output, grammar);
}

auto GrammarToStringComposerUnrestructedGrammar = registration::OverloadRegister < GrammarToStringComposer, void, UnrestrictedGrammar < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerUnrestrictedGrammar = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, UnrestrictedGrammar < > >( GrammarToStringComposer::compose);

}
