/*
 * TreeToStringComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnciek
 */

#ifndef TREE_TO_STRING_COMPOSER_H_
#define TREE_TO_STRING_COMPOSER_H_

#include <ostream>
#include <tree>
#include <core/multipleDispatch.hpp>
#include "tree/Tree.h"
#include "tree/TreeFeatures.h"

#include "alphabet/RankedSymbol.h"

#include <object/ObjectToStringComposer.h>

namespace tree {

/**
 * This class contains methods to print XML representation of tree to the output stream.
 */
class TreeToStringComposer : public std::SingleDispatchFirstStaticParam < TreeToStringComposer, void, std::ostream &, const TreeBase &> {
	template < class SymbolType, class RankType >
	static void compose ( std::ostream &, const std::tree < std::ranked_symbol < SymbolType, RankType > > & node );
	template < class SymbolType, class RankType >
	static void compose ( std::ostream &, const std::ranked_symbol < SymbolType, RankType > & subtreeWildcard, const std::tree < std::ranked_symbol < SymbolType, RankType > > & node );
	template < class SymbolType >
	static void compose ( std::ostream &, const std::tree < SymbolType > & node );
	template < class SymbolType >
	static void compose ( std::ostream &, const SymbolType & subtreeWildcard, const std::tree < SymbolType > & node );

public:
	template < class SymbolType, class RankType >
	static void compose ( std::ostream &, const RankedTree < SymbolType, RankType > & tree );
	template < class SymbolType, class RankType >
	static void compose ( std::ostream &, const RankedPattern < SymbolType, RankType > & tree );
	template < class SymbolType >
	static void compose ( std::ostream &, const UnrankedTree < SymbolType > & tree );
	template < class SymbolType >
	static void compose ( std::ostream &, const UnrankedPattern < SymbolType > & tree );

	/**
	 * Prints XML representation of String to the output stream.
	 * @param tree String to print
	 * @param out output stream to which print the String
	 */
	static void compose ( std::ostream & output, const Tree & tree );

};

template < class SymbolType, class RankType >
void TreeToStringComposer::compose ( std::ostream & out, const std::tree < std::ranked_symbol < SymbolType, RankType> > & node ) {
	alib::ObjectToStringComposer::compose ( out, node.getData ( ).getSymbol ( ) );

	out << std::to_string ( node.getData ( ).getRank ( ) );

	for ( const std::tree < std::ranked_symbol < SymbolType, RankType > > & child : node ) {
		out << " ";
		compose ( out, child );
	}
}

template < class SymbolType, class RankType >
void TreeToStringComposer::compose ( std::ostream & out, const std::ranked_symbol < SymbolType, RankType > & subtreeWildcard, const std::tree < std::ranked_symbol < SymbolType, RankType > > & node ) {
	if ( node.getData ( ) == subtreeWildcard ) {
		out << "#S";
	} else {
		alib::ObjectToStringComposer::compose ( out, node.getData ( ).getSymbol ( ) );

		out << std::to_string ( node.getData ( ).getRank ( ) );

		for ( const std::tree < std::ranked_symbol < SymbolType, RankType > > & child : node ) {
			out << " ";
			compose ( out, subtreeWildcard, child );
		}
	}
}

template < class SymbolType >
void TreeToStringComposer::compose ( std::ostream & out, const std::tree < SymbolType > & node ) {
	alib::ObjectToStringComposer::compose ( out, node.getData ( ) );

	for ( const std::tree < SymbolType > & child : node ) {
		out << " ";
		compose ( out, child );
	}

	out << " |";
}

template < class SymbolType >
void TreeToStringComposer::compose ( std::ostream & out, const SymbolType & subtreeWildcard, const std::tree < SymbolType > & node ) {
	if ( node.getData ( ) == subtreeWildcard ) {
		out << "#S |";
	} else {
		alib::ObjectToStringComposer::compose ( out, node.getData ( ) );

		for ( const std::tree < SymbolType > & child : node ) {
			out << " ";
			compose ( out, subtreeWildcard, child );
		}

		out << " |";
	}
}

} /* namespace tree */

#endif /* TREE_TO_STRING_COMPOSER_H_ */
