#include <list>
#include "GrammarTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "grammar/Unrestricted/UnrestrictedGrammar.h"
#include "grammar/Unrestricted/ContextPreservingUnrestrictedGrammar.h"

#include "grammar/Regular/RightRG.h"
#include "grammar/Regular/RightLG.h"

#include "grammar/ContextFree/LG.h"
#include "grammar/ContextFree/EpsilonFreeCFG.h"
#include "grammar/ContextFree/CFG.h"
#include "grammar/ContextFree/CNF.h"
#include "grammar/ContextFree/GNF.h"

#include "grammar/ContextSensitive/CSG.h"
#include "grammar/ContextSensitive/NonContractingGrammar.h"

#include <grammar/Grammar.h>

#include "factory/XmlDataFactory.hpp"
#include "factory/StringDataFactory.hpp"

#include "alphabet/LabeledSymbol.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarTest );

void GrammarTest::setUp() {
}

void GrammarTest::tearDown() {
}

void GrammarTest::stringParserTest() {
	{
		std::string input = 	"CNF (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ A -> A A | a,\n"
					"  B -> B B | b,\n"
					"  S -> | B S | S A},\n"
					"S)\n";
		grammar::Grammar grammar = alib::StringDataFactory::fromString (input);

		std::string output = alib::StringDataFactory::toString(grammar);

		std::cout << "\"" << input << "\"" << std::endl << std::endl << "\"" << output << "\"" << std::endl;
		CPPUNIT_ASSERT( input == output );

		grammar::Grammar grammar2 = alib::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
	{
		std::string input = 	"RIGHT_RG (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ A -> a | a A,\n"
					"  B -> b | b B,\n"
					"  S -> | a A | a S | b B},\n"
					"S)\n";
		grammar::Grammar grammar = alib::StringDataFactory::fromString (input);

		std::string output = alib::StringDataFactory::toString(grammar);

		std::cout << "\"" << input << "\"" << std::endl << std::endl << "\"" << output << "\"" << std::endl;
		CPPUNIT_ASSERT( input == output );

		grammar::Grammar grammar2 = alib::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
	{
		std::string input = 	"NON_CONTRACTING_GRAMMAR (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ A A -> B B | a A,\n"
					"  B -> b | b B,\n"
					"  S -> A S | B B | S A},\n"
					"S)\n";
		grammar::Grammar grammar = alib::StringDataFactory::fromString (input);

		std::string output = alib::StringDataFactory::toString(grammar);

		std::cout << "\"" << input << "\"" << std::endl << std::endl << "\"" << output << "\"" << std::endl;
		CPPUNIT_ASSERT( input == output );

		grammar::Grammar grammar2 = alib::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
	{
		std::string input = 	"CSG (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ | B | -> b | b B,\n"
					"  | S | -> A S | B B | S A,\n"
					"  A | A | -> B B | a A},\n"
					"S)\n";
		grammar::Grammar grammar = alib::StringDataFactory::fromString (input);

		std::string output = alib::StringDataFactory::toString(grammar);

		std::cout << "\"" << input << "\"" << std::endl << std::endl << "\"" << output << "\"" << std::endl;
		CPPUNIT_ASSERT( input == output );

		grammar::Grammar grammar2 = alib::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
}

