#include <list>
#include "TreeTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "tree/Tree.h"

#include "factory/StringDataFactory.hpp"

#include "alphabet/Symbol.h"
#include "alphabet/LabeledSymbol.h"
#include "alphabet/BlankSymbol.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( TreeTest, "tree" );
CPPUNIT_TEST_SUITE_REGISTRATION ( TreeTest );

void TreeTest::setUp ( ) {
}

void TreeTest::tearDown ( ) {
}

void TreeTest::testEqual ( ) {
	{
		std::string input = "a0";
		tree::Tree tree = alib::StringDataFactory::fromString ( input );

		std::string output = alib::StringDataFactory::toString ( tree );

		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = alib::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "a |";
		tree::Tree tree = alib::StringDataFactory::fromString ( input );

		std::string output = alib::StringDataFactory::toString ( tree );

		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = alib::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "a2 a0 a1 a0";
		tree::Tree tree = alib::StringDataFactory::fromString ( input );

		std::string output = alib::StringDataFactory::toString ( tree );

		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = alib::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "a a | a a | | |";
		tree::Tree tree = alib::StringDataFactory::fromString ( input );

		std::string output = alib::StringDataFactory::toString ( tree );

		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = alib::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "a2 #S a1 a0";
		tree::Tree tree = alib::StringDataFactory::fromString ( input );

		std::string output = alib::StringDataFactory::toString ( tree );

		std::cout << output << std::endl;
		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = alib::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "a #S | a a | | |";
		tree::Tree tree = alib::StringDataFactory::fromString ( input );

		std::string output = alib::StringDataFactory::toString ( tree );

		std::cout << output << std::endl;
		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = alib::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
}
