/*
 * LabelFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelFromStringParser.h"
#include "exception/CommonException.h"
#include "label/PrimitiveLabel.h"
#include "label/ObjectLabel.h"
#include <label/Label.h>

#include <StringApi.hpp>

namespace label {

Label LabelFromStringParser::parseLabel(std::istream& input) {
	return parseLabel(input, std::set<FEATURES>({FEATURES::PRIMITIVE, FEATURES::HEXAVIGESIMAL, FEATURES::OBJECT, FEATURES::UNIQUE_LABEL}));
}

Label LabelFromStringParser::parseLabel(std::istream& input, const std::set<FEATURES>& features) {
	LabelFromStringLexer::Token token = LabelFromStringLexer::next(input);
	switch(token.type) {
	case LabelFromStringLexer::TokenType::TEOF:
		throw exception::CommonException("Unexpected end of file");
	case LabelFromStringLexer::TokenType::ERROR:
		if(!features.count(FEATURES::PRIMITIVE)) throw exception::CommonException("Disabled formalism PrimitiveLabel");
		LabelFromStringLexer::putback(input, token);
		return Label(PrimitiveLabel(alib::stringApi<primitive::Primitive>::parse(input)));
	}
	throw exception::CommonException("Unrecognised input \"" + token.value + "\"");
}

} /* namespace label */
