/*
 * PrimitiveToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <object/ObjectToStringComposer.h>

#include "PrimitiveToStringComposer.h"
#include "primitive/Integer.h"
#include "primitive/String.h"
#include "primitive/Character.h"
#include "primitive/Unsigned.h"
#include "primitive/Bool.h"

#include <registration/AlgoRegistration.hpp>

namespace primitive {

void PrimitiveToStringComposer::compose(std::ostream& out, const String& primitive) {
	auto replace = [](std::string& str, const std::string& what, const std::string& with) {
		size_t index = 0;
		while((index = str.find(what, index)) != std::string::npos) {
			str.replace(index, what.length(), with);
			index += with.length();
		}
	};
	
	std::string tmp = primitive.getData();
	replace(tmp, "'", "\\'" );
	out << '\'' << tmp << '\'';
}

auto PrimitiveToStringComposerString = registration::OverloadRegister < PrimitiveToStringComposer, void, String>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Character& primitive) {
	out << primitive.getData();
}

auto PrimitiveToStringComposerCharacter = registration::OverloadRegister < PrimitiveToStringComposer, void, Character>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Integer& primitive) {
	out << primitive.getData();
}

auto PrimitiveToStringComposerInteger = registration::OverloadRegister < PrimitiveToStringComposer, void, Integer>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Unsigned& primitive) {
	out << primitive.getData();
}

auto PrimitiveToStringComposerUnsigned = registration::OverloadRegister < PrimitiveToStringComposer, void, Unsigned>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Bool& primitive) {
	out << (primitive.getData() ? "true" : "false");
}

auto PrimitiveToStringComposerBool = registration::OverloadRegister < PrimitiveToStringComposer, void, Bool>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Primitive& primitive) {
	dispatch(out, primitive.getData());
}

} /* namespace primitive */
