/*
 * Author: Radovan Cerveny
 */

#include "exception/CommonException.h"
#include "sax/SaxParseInterface.h"
#include <global/GlobalData.h>
#include <tclap/CmdLine.h>

#include <measure>
#include <measurepp>

int main ( int argc, char * * argv ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Measurement provisioner binary", ' ', "0.01" );

		TCLAP::UnlabeledValueArg < std::string > input ( "input", "provisioning file", false, "-", "file" );
		cmd.add ( input );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;

		std::deque < sax::Token > tokens;

		if ( input.getValue ( ) == "-" )
			sax::SaxParseInterface::parseStdin ( tokens );
		else
			sax::SaxParseInterface::parseFile ( input.getValue ( ), tokens );

		auto configuration = measurements::MeasurementProvisionerConfigurationXml::parse ( tokens );
		auto results = measurements::MeasurementProvisioner::runConfiguration ( configuration );

		std::cout << results << std::endl;

		return 0;

	} catch ( const exception::CommonException & exception ) {
		std::cerr << "CommonException caught: " << exception.what ( ) << std::endl << exception.getCause() << std::endl;
		return 4;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
