/*
 * Prompt.cpp
 *
 *  Created on: 20. 7. 2017
 *	  Author: Jan Travnicek
 */

#include "Prompt.h"

#include <cstdlib>
#include <cctype>
#include <cstring>

#include <readline/readline.h>
#include <readline/history.h>

#include <lexer/Lexer.h>
#include <parser/Parser.h>

char * Prompt::stripwhite ( char * begin ) {
	while ( isspace ( * begin ) )
		++ begin;

	char * end = begin + strlen ( begin ) - 1;
	while ( end >= begin && isspace ( * end ) )
		-- end;

	if ( end >= begin )
		* ++ end = '\0';

	return begin;
}

Prompt::Prompt ( cli::Environment environment ) : m_prefix ( "> " ), m_active ( true ), m_exec_result ( 0 ), m_environment ( std::move ( environment ) ) {
}

int Prompt::run ( ) {
	while ( m_active ) {
		char * line = readline ( m_prefix.c_str ( ) );

		if ( ! line )
			continue;

		char * s = stripwhite ( line );

		if ( * s ) {
			add_history ( s );
			execute_line ( std::string ( s ) );
		}

		free ( line );
	}
	return m_exec_result;
}

int Prompt::execute_line ( std::string line ) {
	if ( line == "quit" ) {
		m_active = false;
		m_exec_result = 0;
	} else {
		cli::Parser parser = cli::Parser ( cli::Lexer ( line ) );
		parser.execute ( )->translateAndEval ( nullptr, m_environment );
	}
	return m_exec_result;
}
